# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "adagioxml"


def test_encode_order(self):
    self.configure(adagio_hospitalization_code="1111")
    message = self.order()
    expected = """
    <?xml version="1.0" encoding="UTF-8"?>
        <data>
            <patients>
            <patient id="1">
                <patient_id>L036666666</patient_id>
                <name>
                <lastname>NYME</lastname>
                <firstname>ANO</firstname>
                </name>
                <birthdate>20110513</birthdate>
                <sex>1</sex>
                <samples>
                <sample id="1">
                    <sample_code>123456</sample_code>
                    <sample_type_code>A</sample_type_code>
                    <exam_date>20130603</exam_date>
                    <sample_code>123456</sample_code>
                    <sample_type_code>B</sample_type_code>
                    <exam_date>20130603</exam_date>
                    <sample_prescriber>
                    <sample_prescriber_code>PRESCRIBER</sample_prescriber_code>
                    <sample_prescriber_lastname>Prescriber</sample_prescriber_lastname>
                    </sample_prescriber>
                    <analysiss>
                    <analysis id="1">
                        <germ>
                            <germ_code>1</germ_code>
                            <germ_rank>1</germ_rank>
                        </germ>
                    </analysis>
                    <analysis id="2">
                        <germ>
                            <germ_code>2</germ_code>
                            <germ_rank>2</germ_rank>
                        </germ>
                    </analysis>
                </analysiss>
                </sample>
                </samples>
                <hospitalizations>
                <hospitalization id="1">
                    <hospitalization_code>ADM_CLI</hospitalization_code>
                    <hospitalization_date>
                        <start_hospitalization_date>20130603</start_hospitalization_date>
                    </hospitalization_date>
                    <wards>
                        <ward id="1">
                            <ward_code>LOC</ward_code>
                        </ward>
                    </wards>
                </hospitalization>
                </hospitalizations>
            </patient>
            </patients>
        </data>
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
        <data>
          <query>
            <sample>123456</sample>
          </query>
        </data>
        """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123456"}],
    }
    assert worklist == expected


def test_decode_result_with_diameter(self):
    self.configure(adagio_integration_bmr=True, adagio_get_diametre=True)
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
        <data>
            <header_record>
                <record_type_id>H</record_type_id>
                <name>ADAGIO</name>
            </header_record>
            <patients>
                <patient id="1">
                    <patient_id>78492</patient_id>
                    <name>
                        <lastname>DUPOND</lastname>
                        <firstname>Donald</firstname>
                    </name>
                    <samples>
                        <sample id="1">
                            <sample_code>12345678</sample_code>
                            <sample_date>20170116</sample_date>
                            <sample_type_code>ABC</sample_type_code>
                            <analysiss nb="1">
                                <analysis id="1">
                                    <exam_code>ABC</exam_code>
                                    <exam_date>20170106</exam_date>
                                    <exam_time>121000</exam_time>
                                    <germ>
                                        <germ_code>STAU</germ_code>
                                        <germ_rank>1</germ_rank>
                                    </germ>
                                    <exam_validation_date>20170113</exam_validation_date>
                                    <exam_operator>
                                        <exam_operator_lastname>ADMIN</exam_operator_lastname>
                                        <exam_operator_firstname>ADMIN</exam_operator_firstname>
                                    </exam_operator>
                                    <results nb="2">
                                        <result id="5">
                                          <antibiotic_code>P1</antibiotic_code>
                                          <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                          <result_type>DIA</result_type>
                                          <diameter>21</diameter>
                                        </result>
                                        <result id="5">
                                          <antibiotic_code>P1</antibiotic_code>
                                          <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                          <result_type>RIS</result_type>
                                          <raw_RIS>S</raw_RIS>
                                          <expert_RIS/>
                                          <biologist_RIS/>
                                          <final_RIS>S</final_RIS>
                                        </result>
                                        <result id="5">
                                          <antibiotic_code>P1</antibiotic_code>
                                          <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                          <result_type>MIC</result_type>
                                          <estimated_MIC>&lt;=0.5</estimated_MIC>
                                          <measured_MIC/>
                                        </result>
                                        <result id="6">
                                            <antibiotic_code>FA</antibiotic_code>
                                            <result_type>RIS</result_type>
                                            <final_RIS>S</final_RIS>
                                        </result>
                                        <result id="6">
                                            <antibiotic_code>FA</antibiotic_code>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC>&lt;=1</estimated_MIC>
                                            <measured_MIC></measured_MIC>
                                        </result>
                                        <result id="7">
                                            <antibiotic_code>RIF</antibiotic_code>
                                            <result_type>RIS</result_type>
                                            <final_RIS>R</final_RIS>
                                        </result>
                                        <result id="7">
                                            <antibiotic_code>RIF</antibiotic_code>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC>16</estimated_MIC>
                                            <measured_MIC></measured_MIC>
                                        </result>
                                        <result id="8">
                                             <antibiotic_code>TE</antibiotic_code>
                                             <antibiotic_label>Tetracycline 30 Âµg</antibiotic_label>
                                             <result_type>RIS</result_type>
                                             <raw_RIS/>
                                             <expert_RIS/>
                                             <biologist_RIS/>
                                             <final_RIS/>
                                        </result>
                                        <result id="8">
                                            <antibiotic_code>TE</antibiotic_code>
                                            <antibiotic_label>Tetracycline 30 Âµg</antibiotic_label>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC/>
                                            <measured_MIC/>
                                        </result>
                                        <result id="9">
                                            <result_type>PHE</result_type>
                                            <phenotype_code>BLSE</phenotype_code>
                                        </result>
                                        <result id="12">
                                            <antibiotic_code>CAZ</antibiotic_code>
                                            <result_type>DIA</result_type>
                                            <diameter>21</diameter>
                                        </result>
                                        <result id="12">
                                            <antibiotic_code>CAZ</antibiotic_code>
                                            <result_type>RIS</result_type>
                                            <final_RIS>I</final_RIS>
                                        </result>
                                        <result id="12">
                                            <antibiotic_code>CAZ</antibiotic_code>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC>2</estimated_MIC>
                                            <measured_MIC/>
                                        </result>
                                        <result id="13">
                                            <result_type>DIA</result_type>
                                            <diameter>6</diameter>
                                        </result>
                                        <result id="13">
                                            <result_type>RIS</result_type>
                                            <final_RIS/>
                                        </result>
                                        <result id="13">
                                            <result_type>MIC</result_type>
                                            <estimated_MIC/>
                                            <measured_MIC/>
                                        </result>
                                    </results>
                                    <exam_comments nb="0"/>
                                    <expert_comments nb="3">
                                        <comment id="1" global_id="1">
                                            <comment_source>CC</comment_source>
                                            <comment_code>EUML05</comment_code>
                                            <comment_desc>Description 1</comment_desc>
                                        </comment>
                                        <comment id="2" global_id="2">
                                            <comment_source>CC</comment_source>
                                            <comment_code>EUML08</comment_code>
                                            <comment_desc>Description 2</comment_desc>
                                        </comment>
                                        <comment id="3" global_id="3">
                                            <comment_source>CB</comment_source>
                                            <comment_code/>
                                            <comment_desc>Description 3</comment_desc>
                                        </comment>
                                    </expert_comments>
                                </analysis>
                            </analysiss>
                        </sample>
                    </samples>
                </patient>
            </patients>
        </data>
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "COMMENT_CODE", "value": ["EUML05", "EUML08"]},
                    {
                        "code": "ABC",
                        "comments": ["EUML05", "EUML08"],
                        "date": datetime.datetime(2017, 1, 13, 0, 0),
                        "value": [
                            {
                                "antibiogram": [
                                    {
                                        "antibiotic": "A_P1",
                                        "diametre": "21",
                                        "mic": "<=0.5",
                                        "sensitivity": "S",
                                    },
                                    {
                                        "antibiotic": "A_FA",
                                        "diametre": None,
                                        "mic": "<=1",
                                        "sensitivity": "S",
                                    },
                                    {
                                        "antibiotic": "A_RIF",
                                        "diametre": None,
                                        "mic": "16",
                                        "sensitivity": "R",
                                    },
                                    {
                                        "antibiotic": "BMR_BLSE",
                                        "diametre": None,
                                        "mic": "#",
                                        "sensitivity": "BMR",
                                    },
                                    {
                                        "antibiotic": "A_CAZ",
                                        "diametre": "21",
                                        "mic": "2",
                                        "sensitivity": "I",
                                    },
                                ],
                                "sub_code": "1",
                                "value": "G_STAU",
                            }
                        ],
                    },
                ],
                "sample_id": "12345678",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_measured_mic(self):
    self.configure(adagio_cmi_measured=True, adagio_phenotype="BMR=Phenotype BMR")
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
        <data>
            <header_record>
                <record_type_id>H</record_type_id>
                <name>ADAGIO</name>
            </header_record>
            <patients nb="1">
                <patient id="1">
                    <record_type_id>P</record_type_id>
                    <sequence_number>1</sequence_number>
                    <patient_id>8987848745</patient_id>
                    <name>
                        <lastname>z_labo</lastname>
                        <firstname>IQ20011</firstname>
                    </name>
                    <samples nb="1">
                        <sample id="1">
                            <sample_code>180401</sample_code>
                            <sample_type_code>BOEIL</sample_type_code>
                            <analysiss nb="1">
                                <analysis id="1">
                                    <exam_code>A-1</exam_code>
                                    <exam_date>20190418</exam_date>
                                    <germ>
                                        <germ_code>ESCCOL</germ_code>
                                        <germ_rank>1</germ_rank>
                                    </germ>
                                    <exam_validation_date>20190418</exam_validation_date>
                                    <results nb="13">
                                        <result id="3">
                                            <antibiotic_code>CMIAMC</antibiotic_code>
                                            <result_type>DIA</result_type>
                                            <diameter/>
                                        </result>
                                        <result id="3">
                                            <antibiotic_code>CMIAMC</antibiotic_code>
                                            <result_type>RIS</result_type>
                                            <raw_RIS>S</raw_RIS>
                                            <expert_RIS/>
                                            <biologist_RIS/>
                                            <final_RIS>S</final_RIS>
                                        </result>
                                        <result id="3">
                                            <antibiotic_code>CMIAMC</antibiotic_code>
                                            <result_type>MIC</result_type>
                                            <measured_MIC>4</measured_MIC>
                                        </result>
                                        <result id="4">
                                            <antibiotic_code>AMC</antibiotic_code>
                                            <result_type>DIA</result_type>
                                            <diameter/>
                                        </result>
                                        <result id="4">
                                            <antibiotic_code>AMC</antibiotic_code>
                                            <result_type>RIS</result_type>
                                            <raw_RIS/>
                                            <expert_RIS>S</expert_RIS>
                                            <biologist_RIS/>
                                            <final_RIS>S</final_RIS>
                                        </result>
                                        <result id="4">
                                            <antibiotic_code>AMC</antibiotic_code>
                                            <result_type>MIC</result_type>
                                            <measured_MIC/>
                                        </result>
                                        <result id="4">
                                            <result_type>PHE</result_type>
                                            <phenotype_code>BMR</phenotype_code>
                                        </result>
                                    </results>
                                </analysis>
                            </analysiss>
                        </sample>
                    </samples>
                </patient>
            </patients>
        </data>
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "BOEIL",
                        "comments": [],
                        "date": datetime.datetime(2019, 4, 18, 0, 0),
                        "phenotypes": ["BMR"],
                        "value": [
                            {
                                "antibiogram": [
                                    {
                                        "antibiotic": "A_CMIAMC",
                                        "mic": "4",
                                        "sensitivity": "S",
                                    },
                                    {
                                        "antibiotic": "A_AMC",
                                        "mic": None,
                                        "sensitivity": "S",
                                    },
                                ],
                                "sub_code": "1",
                                "value": "G_ESCCOL",
                            }
                        ],
                    }
                ],
                "sample_id": "180401",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_ignore_result_atb_without_cmi_and_ris(self):
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
        <data>
            <header_record>
                <record_type_id>H</record_type_id>
                <name>ADAGIO</name>
            </header_record>
            <patients>
                <patient id="1">
                    <patient_id>78492</patient_id>
                    <name>
                        <lastname>ESSAI</lastname>
                        <firstname>BACTERIO</firstname>
                    </name>
                    <samples>
                        <sample id="1">
                            <sample_code>12345678</sample_code>
                            <sample_date>20170116</sample_date>
                            <sample_type_code>ABC</sample_type_code>
                            <analysiss nb="1">
                                <analysis id="1">
                                    <exam_code>ABC</exam_code>
                                    <exam_date>20170106</exam_date>
                                    <exam_time>121000</exam_time>
                                    <germ>
                                        <germ_code>STAU</germ_code>
                                        <germ_rank>1</germ_rank>
                                    </germ>
                                    <exam_validation_date>20170113</exam_validation_date>
                                    <exam_operator>
                                        <exam_operator_lastname>ADMIN</exam_operator_lastname>
                                        <exam_operator_firstname>ADMIN</exam_operator_firstname>
                                    </exam_operator>
                                    <results nb="2">
                                        <result id="1">
                                            <antibiotic_code>P1</antibiotic_code>
                                            <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                            <result_type>DIA</result_type>
                                            <diameter>21</diameter>
                                        </result>
                                        <result id="1">
                                            <antibiotic_code>P1</antibiotic_code>
                                            <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                            <result_type>RIS</result_type>
                                            <raw_RIS>S</raw_RIS>
                                            <expert_RIS/>
                                            <biologist_RIS/>
                                            <final_RIS>S</final_RIS>
                                        </result>
                                        <result id="1">
                                            <antibiotic_code>P1</antibiotic_code>
                                            <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC>&lt;=0.5</estimated_MIC>
                                            <measured_MIC/>
                                        </result>
                                        <result id="2">
                                            <antibiotic_code>CC</antibiotic_code>
                                            <antibiotic_label>Clindamycin 2</antibiotic_label>
                                            <result_type>DIA</result_type>
                                            <diameter>21</diameter>
                                        </result>
                                        <result id="2">
                                            <antibiotic_code>CC</antibiotic_code>
                                            <antibiotic_label>Clindamycin 2</antibiotic_label>
                                            <result_type>RIS</result_type>
                                            <final_RIS/>
                                        </result>
                                        <result id="2">
                                            <antibiotic_code>CC</antibiotic_code>
                                            <antibiotic_label>Clindamycin 2</antibiotic_label>
                                            <result_type>MIC</result_type>
                                            <measured_MIC/>
                                        </result>
                                    </results>
                                </analysis>
                            </analysiss>
                        </sample>
                    </samples>
                </patient>
            </patients>
        </data>
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "ABC",
                        "comments": [],
                        "date": datetime.datetime(2017, 1, 13, 0, 0),
                        "value": [
                            {
                                "antibiogram": [
                                    {
                                        "antibiotic": "A_P1",
                                        "mic": "<=0.5",
                                        "sensitivity": "S",
                                    }
                                ],
                                "sub_code": "1",
                                "value": "G_STAU",
                            }
                        ],
                    }
                ],
                "sample_id": "12345678",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_bmr(self):
    self.configure(adagio_integration_bmr=True)
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
        <data>
            <header_record>
                <record_type_id>H</record_type_id>
                <name>ADAGIO</name>
            </header_record>
            <patients>
                <patient id="1">
                    <patient_id>78492</patient_id>
                    <name>
                        <lastname>DUPOND</lastname>
                        <firstname>Donald</firstname>
                    </name>
                    <samples>
                        <sample id="1">
                            <sample_code>12345678</sample_code>
                            <sample_date>20170116</sample_date>
                            <sample_type_code>ABC</sample_type_code>
                            <analysiss nb="1">
                                <analysis id="1">
                                    <exam_code>ABC</exam_code>
                                    <exam_date>20170106</exam_date>
                                    <exam_time>121000</exam_time>
                                    <germ>
                                        <germ_code>STAU</germ_code>
                                        <germ_rank>1</germ_rank>
                                    </germ>
                                    <exam_validation_date>20170113</exam_validation_date>
                                    <exam_operator>
                                        <exam_operator_lastname>ADMIN</exam_operator_lastname>
                                        <exam_operator_firstname>ADMIN</exam_operator_firstname>
                                    </exam_operator>
                                    <results nb="2">
                                        <result id="5">
                                          <antibiotic_code>P1</antibiotic_code>
                                          <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                          <result_type>DIA</result_type>
                                          <diameter>21</diameter>
                                        </result>
                                        <result id="5">
                                          <antibiotic_code>P1</antibiotic_code>
                                          <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                          <result_type>RIS</result_type>
                                          <raw_RIS>S</raw_RIS>
                                          <expert_RIS/>
                                          <biologist_RIS/>
                                          <final_RIS>S</final_RIS>
                                        </result>
                                        <result id="5">
                                          <antibiotic_code>P1</antibiotic_code>
                                          <antibiotic_label>Benzylpenicillin 1 IU</antibiotic_label>
                                          <result_type>MIC</result_type>
                                          <estimated_MIC>&lt;=0.5</estimated_MIC>
                                          <measured_MIC/>
                                        </result>
                                        <result id="6">
                                            <antibiotic_code>FA</antibiotic_code>
                                            <result_type>RIS</result_type>
                                            <final_RIS>S</final_RIS>
                                        </result>
                                        <result id="6">
                                            <antibiotic_code>FA</antibiotic_code>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC>&lt;=1</estimated_MIC>
                                            <measured_MIC></measured_MIC>
                                        </result>
                                        <result id="7">
                                            <antibiotic_code>RIF</antibiotic_code>
                                            <result_type>RIS</result_type>
                                            <final_RIS>R</final_RIS>
                                        </result>
                                        <result id="7">
                                            <antibiotic_code>RIF</antibiotic_code>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC>16</estimated_MIC>
                                            <measured_MIC></measured_MIC>
                                        </result>
                                        <result id="8">
                                             <antibiotic_code>TE</antibiotic_code>
                                             <antibiotic_label>Tetracycline 30 Âµg</antibiotic_label>
                                             <result_type>RIS</result_type>
                                             <raw_RIS/>
                                             <expert_RIS/>
                                             <biologist_RIS/>
                                             <final_RIS/>
                                        </result>
                                        <result id="8">
                                            <antibiotic_code>TE</antibiotic_code>
                                            <antibiotic_label>Tetracycline 30 Âµg</antibiotic_label>
                                            <result_type>MIC</result_type>
                                            <estimated_MIC/>
                                            <measured_MIC/>
                                        </result>
                                        <result id="9">
                                            <result_type>PHE</result_type>
                                            <phenotype_code>BLSE</phenotype_code>
                                        </result>
                                    </results>
                                    <exam_comments nb="0"/>
                                    <expert_comments nb="3">
                                        <comment id="1" global_id="1">
                                            <comment_source>CC</comment_source>
                                            <comment_code>EUML05</comment_code>
                                            <comment_desc>Description 1</comment_desc>
                                        </comment>
                                        <comment id="2" global_id="2">
                                            <comment_source>CC</comment_source>
                                            <comment_code>EUML08</comment_code>
                                            <comment_desc>Description 2</comment_desc>
                                        </comment>
                                        <comment id="3" global_id="3">
                                            <comment_source>CB</comment_source>
                                            <comment_code/>
                                            <comment_desc>Description 3</comment_desc>
                                        </comment>
                                    </expert_comments>
                                </analysis>
                            </analysiss>
                        </sample>
                    </samples>
                </patient>
            </patients>
        </data>
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "COMMENT_CODE", "value": ["EUML05", "EUML08"]},
                    {
                        "code": "ABC",
                        "comments": ["EUML05", "EUML08"],
                        "date": datetime.datetime(2017, 1, 13, 0, 0),
                        "value": [
                            {
                                "antibiogram": [
                                    {
                                        "antibiotic": "A_P1",
                                        "mic": "<=0.5",
                                        "sensitivity": "S",
                                    },
                                    {
                                        "antibiotic": "A_FA",
                                        "mic": "<=1",
                                        "sensitivity": "S",
                                    },
                                    {
                                        "antibiotic": "A_RIF",
                                        "mic": "16",
                                        "sensitivity": "R",
                                    },
                                    {
                                        "antibiotic": "BMR_BLSE",
                                        "mic": "#",
                                        "sensitivity": "BMR",
                                    },
                                ],
                                "sub_code": "1",
                                "value": "G_STAU",
                            }
                        ],
                    },
                ],
                "sample_id": "12345678",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_pheno_without_cmi(self):
    self.configure(adagio_get_diametre=True)
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
            <data>
                <header_record>
                    <record_type_id>H</record_type_id>
                    <delimiter_definition>\\^&amp;</delimiter_definition>
                    <name>ADAGIO</name>
                    <generic1/>
                    <generic2/>
                    <config>
                    <CaId>2</CaId>
                    <CaFeel>T</CaFeel>
                    <CaAutomat>LIS</CaAutomat>
                    <CaEnvironment/>
                    </config>
                </header_record>
                <patients nb="1">
                    <patient id="1">
                    <record_type_id>P</record_type_id>
                    <sequence_number>1</sequence_number>
                    <patient_id>123</patient_id>
                    <name>
                        <lastname>TEST</lastname>
                        <firstname>ISABELLE</firstname>
                    </name>
                    <birthdate>20070101</birthdate>
                    <sex>1</sex>
                    <patient_details/>
                    <treatments nb="0"/>
                    <hospitalizations nb="1">
                        <hospitalization id="1">
                        <hospitalization_date/>
                        <stay/>
                        <hospitalization_reason/>
                        <wards nb="0"/>
                        </hospitalization>
                    </hospitalizations>
                    <samples nb="1">
                        <sample id="1">
                        <sample_code>1234</sample_code>
                        <sample_prescriber/>
                        <sample_type_code>CBU3</sample_type_code>
                        <analysiss nb="1">
                            <analysis id="1">
                                <exam_code>A-1</exam_code>
                                <exam_date>20190604</exam_date>
                                <exam_prescriber/>
                                <germ>
                                    <germ_code>citfre</germ_code>
                                    <germ_rank>1</germ_rank>
                                </germ>
                                <exam_validation_date>20190604</exam_validation_date>
                                <mtyping/>
                                <results nb="9">
                                    <result id="3">
                                        <antibiotic_code>TZP</antibiotic_code>
                                        <result_type>DIA</result_type>
                                        <diameter>16</diameter>
                                    </result>
                                    <result id="3">
                                        <antibiotic_code>TZP</antibiotic_code>
                                        <result_type>RIS</result_type>
                                        <raw_RIS>R</raw_RIS>
                                        <expert_RIS/>
                                        <biologist_RIS/>
                                        <final_RIS>R</final_RIS>
                                    </result>
                                    <result id="3">
                                        <antibiotic_code>TZP</antibiotic_code>
                                        <result_type>MIC</result_type>
                                        <measured_MIC/>
                                    </result>
                                    <result id="4">
                                        <antibiotic_code>MEC</antibiotic_code>
                                        <result_type>DIA</result_type>
                                        <diameter>35</diameter>
                                    </result>
                                    <result id="4">
                                        <antibiotic_code>MEC</antibiotic_code>
                                        <result_type>RIS</result_type>
                                        <raw_RIS/>
                                        <expert_RIS/>
                                        <biologist_RIS/>
                                        <final_RIS/>
                                    </result>
                                    <result id="4">
                                        <antibiotic_code>MEC</antibiotic_code>
                                        <result_type>MIC</result_type>
                                        <measured_MIC/>
                                    </result>
                                    <result id="5">
                                        <antibiotic_code>TAX</antibiotic_code>
                                        <result_type>DIA</result_type>
                                        <diameter>12</diameter>
                                    </result>
                                    <result id="5">
                                        <antibiotic_code>TAX</antibiotic_code>
                                        <result_type>RIS</result_type>
                                        <raw_RIS>R</raw_RIS>
                                        <expert_RIS/>
                                        <biologist_RIS/>
                                        <final_RIS>R</final_RIS>
                                    </result>
                                    <result id="5">
                                        <antibiotic_code>TAX</antibiotic_code>
                                        <result_type>MIC</result_type>
                                        <measured_MIC/>
                                    </result>
                                    <result id="6">
                                        <antibiotic_code>CTR</antibiotic_code>
                                        <result_type>DIA</result_type>
                                        <diameter>18</diameter>
                                    </result>
                                    <result id="6">
                                        <antibiotic_code>CTR</antibiotic_code>
                                        <result_type>RIS</result_type>
                                        <raw_RIS>R</raw_RIS>
                                        <expert_RIS/>
                                        <biologist_RIS/>
                                        <final_RIS>R</final_RIS>
                                    </result>
                                    <result id="6">
                                        <antibiotic_code>CTR</antibiotic_code>
                                        <result_type>MIC</result_type>
                                        <measured_MIC/>
                                    </result>
                                    <result id="7">
                                        <antibiotic_code>TAZ</antibiotic_code>
                                        <result_type>DIA</result_type>
                                        <diameter>15</diameter>
                                    </result>
                                    <result id="7">
                                        <antibiotic_code>TAZ</antibiotic_code>
                                        <result_type>RIS</result_type>
                                        <raw_RIS>R</raw_RIS>
                                        <expert_RIS/>
                                        <biologist_RIS/>
                                        <final_RIS>R</final_RIS>
                                    </result>
                                    <result id="7">
                                        <antibiotic_code>TAZ</antibiotic_code>
                                        <result_type>MIC</result_type>
                                        <measured_MIC/>
                                    </result>
                                    <result id="8">
                                        <antibiotic_code>FEP</antibiotic_code>
                                        <result_type>DIA</result_type>
                                        <diameter>35</diameter>
                                    </result>
                                    <result id="8">
                                        <antibiotic_code>FEP</antibiotic_code>
                                        <result_type>RIS</result_type>
                                        <raw_RIS>S</raw_RIS>
                                        <expert_RIS/>
                                        <biologist_RIS/>
                                        <final_RIS>S</final_RIS>
                                    </result>
                                    <result id="8">
                                        <antibiotic_code>FEP</antibiotic_code>
                                        <result_type>MIC</result_type>
                                        <measured_MIC/>
                                    </result>
                                    <result id="1">
                                        <result_type>PHE</result_type>
                                        <phenotype_code>HCASE</phenotype_code>
                                    </result>
                                    <result id="2">
                                        <result_type>PHE</result_type>
                                        <phenotype_code/>
                                    </result>
                                </results>
                            </analysis>
                        </analysiss>
                        </sample>
                    </samples>
                    </patient>
                </patients>
                <terminator_record>
                    <record_type_id>L</record_type_id>
                    <sequence_number>1</sequence_number>
                    <codif_code>F</codif_code>
                </terminator_record>
            </data>
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "CBU3",
                        "comments": [],
                        "date": datetime.datetime(2019, 6, 4, 0, 0),
                        "phenotypes": ["HCASE"],
                        "value": [
                            {
                                "antibiogram": [
                                    {
                                        "antibiotic": "A_TZP",
                                        "diametre": "16",
                                        "mic": None,
                                        "sensitivity": "R",
                                    },
                                    {
                                        "antibiotic": "A_TAX",
                                        "diametre": "12",
                                        "mic": None,
                                        "sensitivity": "R",
                                    },
                                    {
                                        "antibiotic": "A_CTR",
                                        "diametre": "18",
                                        "mic": None,
                                        "sensitivity": "R",
                                    },
                                    {
                                        "antibiotic": "A_TAZ",
                                        "diametre": "15",
                                        "mic": None,
                                        "sensitivity": "R",
                                    },
                                    {
                                        "antibiotic": "A_FEP",
                                        "diametre": "35",
                                        "mic": None,
                                        "sensitivity": "S",
                                    },
                                ],
                                "sub_code": "1",
                                "value": "G_CITFRE",
                            }
                        ],
                    }
                ],
                "sample_id": "1234",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_no_atb(self):
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
            <data>
            <header_record>
                <record_type_id>H</record_type_id>
                <delimiter_definition>\\^&amp;</delimiter_definition>
                <name>ADAGIO</name>
                <generic1/>
                <generic2/>
                <config>
                <CaId>2</CaId>
                <CaFeel>T</CaFeel>
                <CaAutomat>LIS</CaAutomat>
                <CaEnvironment/>
                </config>
            </header_record>
            <patients nb="1">
                <patient id="1">
                <record_type_id>P</record_type_id>
                <sequence_number>1</sequence_number>
                <patient_id>123456789</patient_id>
                <patient_health_id/>
                <name>
                    <lastname>ANO</lastname>
                    <firstname>NYME</firstname>
                </name>
                <title/>
                <birthdate>19500101</birthdate>
                <sex>2</sex>
                <origin/>
                <patient_details>
                    <street/>
                    <city/>
                    <state/>
                    <post_code/>
                    <country/>
                    <phone/>
                    <mobile/>
                    <email/>
                </patient_details>
                <health_status/>
                <treatments nb="0"/>
                <hospitalizations nb="1">
                    <hospitalization id="1">
                    <hospitalization_code>123456789</hospitalization_code>
                    <hospitalization_date>
                        <start_hospitalization_date>20220117</start_hospitalization_date>
                        <end_hospitalization_date/>
                    </hospitalization_date>
                    <stay/>
                    <hospitalization_reason/>
                    <wards nb="0"/>
                    </hospitalization>
                </hospitalizations>
                <samples nb="1">
                    <sample id="1">
                    <sample_code>1234</sample_code>
                    <sample_prescriber/>
                    <sample_date>20220120</sample_date>
                    <sample_time>094218</sample_time>
                    <sample_type_code>URIN</sample_type_code>
                    <analysiss nb="1">
                        <analysis id="1">
                        <exam_date>20220120</exam_date>
                        <exam_time>094218</exam_time>
                        <exam_prescriber/>
                        <germ>
                            <germ_code>ENCL</germ_code>
                            <germ_rank>2</germ_rank>
                        </germ>
                        <exam_validation_date>20220120</exam_validation_date>
                        <exam_operator>
                            <exam_operator_lastname>TECH</exam_operator_lastname>
                            <exam_operator_firstname>NICIAN</exam_operator_firstname>
                        </exam_operator>
                        <mtyping/>
                        <results nb="7">
                            <result id="2">
                                <antibiotic_code>CTXC_U</antibiotic_code>
                                <antibiotic_label>Céfotaxime CLOXA 30 µg</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter>6</diameter>
                            </result>
                            <result id="2">
                                <antibiotic_code>CTXC_U</antibiotic_code>
                                <antibiotic_label>Céfotaxime CLOXA 30 µg</antibiotic_label>
                                <result_type>RIS</result_type>
                                <final_RIS/>
                            </result>
                            <result id="2">
                                <antibiotic_code>CTXC_U</antibiotic_code>
                                <antibiotic_label>Céfotaxime CLOXA 30 µg</antibiotic_label>
                                <result_type>MIC</result_type>
                                <measured_MIC/>
                            </result>
                            <result id="3">
                                <antibiotic_code>CCTXC_U</antibiotic_code>
                                <antibiotic_label>Céfotaxime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter>31</diameter>
                            </result>
                            <result id="3">
                                <antibiotic_code>CCTXC_U</antibiotic_code>
                                <antibiotic_label>Céfotaxime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>RIS</result_type>
                                <final_RIS/>
                            </result>
                            <result id="3">
                                <antibiotic_code>CCTXC_U</antibiotic_code>
                                <antibiotic_label>Céfotaxime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>MIC</result_type>
                                <measured_MIC/>
                            </result>
                            <result id="4">
                                <antibiotic_code>CAZC_U</antibiotic_code>
                                <antibiotic_label>Ceftazidime CLOXA 30 µg</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter>15</diameter>
                            </result>
                            <result id="4">
                                <antibiotic_code>CAZC_U</antibiotic_code>
                                <antibiotic_label>Ceftazidime CLOXA 30 µg</antibiotic_label>
                                <result_type>RIS</result_type>
                                <final_RIS/>
                            </result>
                            <result id="4">
                                <antibiotic_code>CAZC_U</antibiotic_code>
                                <antibiotic_label>Ceftazidime CLOXA 30 µg</antibiotic_label>
                                <result_type>MIC</result_type>
                                <measured_MIC/>
                            </result>
                            <result id="5">
                                <antibiotic_code>CCAZC_U</antibiotic_code>
                                <antibiotic_label>Ceftazidime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter>26</diameter>
                            </result>
                            <result id="5">
                                <antibiotic_code>CCAZC_U</antibiotic_code>
                                <antibiotic_label>Ceftazidime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>RIS</result_type>
                                <final_RIS/>
                            </result>
                            <result id="5">
                                <antibiotic_code>CCAZC_U</antibiotic_code>
                                <antibiotic_label>Ceftazidime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>MIC</result_type>
                                <measured_MIC/>
                            </result>
                            <result id="6">
                                <antibiotic_code>FEPC_U</antibiotic_code>
                                <antibiotic_label>Céfépime CLOXA 30 µg</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter>18</diameter>
                            </result>
                            <result id="6">
                                <antibiotic_code>FEPC_U</antibiotic_code>
                                <antibiotic_label>Céfépime CLOXA 30 µg</antibiotic_label>
                                <result_type>RIS</result_type>
                                <final_RIS/>
                            </result>
                            <result id="6">
                                <antibiotic_code>FEPC_U</antibiotic_code>
                                <antibiotic_label>Céfépime CLOXA 30 µg</antibiotic_label>
                                <result_type>MIC</result_type>
                                <measured_MIC/>
                            </result>
                            <result id="7">
                                <antibiotic_code>CFEPC_U</antibiotic_code>
                                <antibiotic_label>Céfépime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter>30</diameter>
                            </result>
                            <result id="7">
                                <antibiotic_code>CFEPC_U</antibiotic_code>
                                <antibiotic_label>Céfépime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>RIS</result_type>
                                <final_RIS/>
                            </result>
                            <result id="7">
                                <antibiotic_code>CFEPC_U</antibiotic_code>
                                <antibiotic_label>Céfépime + acide clavulanique CLOXA 40 µg</antibiotic_label>
                                <result_type>MIC</result_type>
                                <measured_MIC/>
                            </result>
                            <result id="1">
                                <result_type>PHE</result_type>
                                <phenotype_code>EBLSE</phenotype_code>
                            </result>
                        </results>
                        </analysis>
                    </analysiss>
                    </sample>
                </samples>
                </patient>
            </patients>
            <terminator_record>
                <record_type_id>L</record_type_id>
                <sequence_number>1</sequence_number>
                <codif_code>F</codif_code>
            </terminator_record>
            </data>
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "URIN",
                        "comments": [],
                        "date": datetime.datetime(2022, 1, 20, 0, 0),
                        "phenotypes": ["EBLSE"],
                        "value": [
                            {"antibiogram": [], "sub_code": "2", "value": "G_ENCL"}
                        ],
                    }
                ],
                "sample_id": "1234",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_maine_anjou(self):
    worklist = self.decode(
        """
        <?xml version="1.0" encoding="UTF-8"?>
            <data>
            <header_record>
                <record_type_id>H</record_type_id>
                <delimiter_definition>\\^&amp;</delimiter_definition>
                <name>ADAGIO</name>
                <generic1/>
                <generic2/>
                <config>
                <CaId>2</CaId>
                <CaFeel>T</CaFeel>
                <CaAutomat>LIS</CaAutomat>
                <CaEnvironment/>
                </config>
            </header_record>
            <patients nb="1">
                <patient id="1">
                <record_type_id>P</record_type_id>
                <sequence_number>1</sequence_number>
                <patient_id>170606030075</patient_id>
                <patient_health_id/>
                <name>
                    <lastname>TEST</lastname>
                    <firstname>Michel</firstname>
                </name>
                <title/>
                <birthdate>19500926</birthdate>
                <sex>2</sex>
                <origin/>
                <patient_details>
                    <street/>
                    <city/>
                    <state/>
                    <post_code/>
                    <country/>
                    <phone/>
                    <mobile/>
                    <email/>
                </patient_details>
                <health_status/>
                <treatments nb="0"/>
                <hospitalizations nb="2">
                    <hospitalization id="1">
                    <hospitalization_code>1111</hospitalization_code>
                    <hospitalization_date>
                        <start_hospitalization_date>20220712</start_hospitalization_date>
                        <end_hospitalization_date/>
                    </hospitalization_date>
                    <stay/>
                    <hospitalization_reason/>
                    <wards nb="0"/>
                    </hospitalization>
                    <hospitalization id="2">
                    <hospitalization_code>1111</hospitalization_code>
                    <hospitalization_date>
                        <start_hospitalization_date>20220730</start_hospitalization_date>
                        <end_hospitalization_date/>
                    </hospitalization_date>
                    <stay/>
                    <hospitalization_reason/>
                    <wards nb="0"/>
                    </hospitalization>
                </hospitalizations>
                <samples nb="1">
                    <sample id="1234">
                    <sample_code>1234</sample_code>
                    <sample_prescriber/>
                    <sample_date>20220802</sample_date>
                    <sample_time>104544</sample_time>
                    <sample_type_code>ACRA</sample_type_code>
                    <sample_type_name>EXPECTORATION</sample_type_name>
                    <sample_status>0</sample_status>
                    <analysiss nb="1">
                        <analysis id="1">
                        <exam_code>A-1</exam_code>
                        <exam_date>20220802</exam_date>
                        <exam_time>104544</exam_time>
                        <exam_prescriber/>
                        <germ>
                            <germ_code>STRPNE</germ_code>
                            <germ_name>Streptococcus pneumoniae</germ_name>
                            <germ_rank>1</germ_rank>
                        </germ>
                        <exam_status>2</exam_status>
                        <exam_validation_date>20220803</exam_validation_date>
                        <exam_operator>
                            <exam_operator_lastname>Barré</exam_operator_lastname>
                            <exam_operator_firstname>Anne Laure</exam_operator_firstname>
                        </exam_operator>
                        <mtyping/>
                        <results nb="3">
                            <result id="18">
                                <antibiotic_code>TAX</antibiotic_code>
                                <antibiotic_label>Céfotaxime</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter/>
                            </result>
                            <result id="18">
                                <antibiotic_code>TAX</antibiotic_code>
                                <antibiotic_label>Céfotaxime</antibiotic_label>
                                <result_type>RIS</result_type>
                                <raw_RIS>S</raw_RIS>
                                <expert_RIS/>
                                <biologist_RIS/>
                                <final_RIS>S</final_RIS>
                            </result>
                            <result id="18">
                                <antibiotic_code>TAX</antibiotic_code>
                                <antibiotic_label>Céfotaxime</antibiotic_label>
                                <result_type>MIC</result_type>
                                <estimated_MIC/>
                                <measured_MIC>0.38</measured_MIC>
                            </result>
                                <result id="19">
                                <antibiotic_code/>
                                <antibiotic_label>Pénicilline - Méningites</antibiotic_label>
                                <result_type>DIA</result_type>
                                <diameter/>
                            </result>
                            <result id="19">
                                <antibiotic_code/>
                                <antibiotic_label>Pénicilline - Méningites</antibiotic_label>
                                <result_type>RIS</result_type>
                                <raw_RIS/>
                                <expert_RIS>R</expert_RIS>
                                <biologist_RIS/>
                                <final_RIS>R</final_RIS>
                            </result>
                            <result id="19">
                                <antibiotic_code/>
                                <antibiotic_label>Pénicilline - Méningites</antibiotic_label>
                                <result_type>MIC</result_type>
                                <estimated_MIC/>
                                <measured_MIC/>
                            </result>
                        </results>
                        </analysis>
                    </analysiss>
                    <sample_comments nb="0"/>
                    </sample>
                </samples>
                <patient_comments nb="0"/>
                </patient>
            </patients>
            <terminator_record>
                <record_type_id>L</record_type_id>
                <sequence_number>1</sequence_number>
                <codif_code>F</codif_code>
            </terminator_record>
            </data>
        """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "ACRA",
                        "comments": [],
                        "date": datetime.datetime(2022, 8, 3, 0, 0),
                        "value": [
                            {
                                "antibiogram": [
                                    {
                                        "antibiotic": "A_TAX",
                                        "mic": None,
                                        "sensitivity": "S",
                                    }
                                ],
                                "sub_code": "1",
                                "value": "G_STRPNE",
                            }
                        ],
                    }
                ],
                "sample_id": "1234",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
