# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "autovue"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||OCD^AV2G^1.0^J271408|||||||P|1|20130904133730
    Q|1|^SAMPLE1||||||||||O
    Q|2|^SAMPLE2||||||||||O
    Q|3|^SAMPLE3||||||||||O
    Q|4|^SAMPLE4||||||||||O
    L|1"""
    )
    expected = {
        "context": "query",
        "samples": [
            {"sample_id": "SAMPLE1", "type": "patient"},
            {"sample_id": "SAMPLE2", "type": "patient"},
            {"sample_id": "SAMPLE3", "type": "patient"},
            {"sample_id": "SAMPLE4", "type": "patient"},
        ],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
                H|\\^&|||OCD^AV2G^1.0^J271408|||||||P|1|20130904133730
                Q|1|^SAMPLE1||||||||||O
                L|1"""
            )
        },
    )
    expected = """
    H|\\^&|||AUTOVUE|||||||||20130715141912
    L|1|N"""
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||AUTOVUE|||||||||20130715141912
        P|1|20130513123456||036666666|NYME^ANO|NJF|20110513121500|F
        O|1|123456||A|S|20130513|||||N||||CENTBLOOD
        O|2|123456||B|S|20130513|||||A||||CENTBLOOD
        O|3|123456||C|S|20130513|||||C||||CENTBLOOD
        L|1|N"""
    assert result == expected


def test_order_vision(self):
    self.configure(nom_jeune_fille_dans_champ_p15=True, ppid_en_p3=True)
    result = self.order()
    expected = """H|\\^&|||AUTOVUE|||||||||20130715141912
        P|1|036666666||036666666|NYME^ANO||20110513121500|F||||||NJF
        O|1|123456||A|S|20130513|||||N||||CENTBLOOD
        O|2|123456||B|S|20130513|||||A||||CENTBLOOD
        O|3|123456||C|S|20130513|||||C||||CENTBLOOD
        L|1|N"""
    assert result == expected


def test_result_manuel(self):
    trame = """
    H|\\^&|||OCD^AV2G^1.0^J271408|||||||P|1|20130828151054
    P|1|q308286940685776||030828413627|POTET^SEBASTIEN||19700906000000|M
    O|1|6940685776||GRPH|N|20130828000000|||||||||CENTBLOOD|||||||20130828151021|||F
    R|1|ABO|A|||||F||Automatique^User||20130828151021|J271408
    M|1|Anti-A+B|ABO-Rh^3^084444^11600^20230403235959^03042344084444116006_20221129_170425_BW.jpg^03042344084444116006_20221129_170425_Color.jpg||0^M^40^soladmin|4 ABO(FWD)-D/CDE-44 + (RVS)-A1,B
    L"""
    result = self.decode(trame)
    assert result == {
        "samples": [
            {
                "sample_id": "6940685776",
                "type": "patient",
                "results": [
                    {
                        "code": "ABO",
                        "value": "A",
                        "flags": [],
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "lots": [],
                        "operator": "User",
                        "instrument": "J271408",
                    },
                    {
                        "code": "ABO_Anti-A+B",
                        "value": "ABO-Rh^3^084444^11600^2023-04-03 23:59:59^03042344084444116006_20221129_170425_Color.jpg^0^40",
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "instrument": "J271408",
                        "flags": ["M", "40"],
                    },
                ],
            }
        ],
        "context": "result",
    }
    self.configure(options_correction_manuel=False)
    result1 = self.decode(trame)
    assert result1 == {
        "samples": [
            {
                "sample_id": "6940685776",
                "type": "patient",
                "results": [
                    {
                        "code": "ABO",
                        "value": "A",
                        "flags": [],
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "lots": [],
                        "operator": "User",
                        "instrument": "J271408",
                    },
                    {
                        "code": "ABO_Anti-A+B",
                        "value": "ABO-Rh^3^084444^11600^2023-04-03 23:59:59^03042344084444116006_20221129_170425_Color.jpg^0",
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "instrument": "J271408",
                        "flags": ["M"],
                    },
                ],
            }
        ],
        "context": "result",
    }


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||OCD^AV2G^1.0^J271408|||||||P|1|20130828151054
    P|1|q308286940685776||030828413627|POTET^SEBASTIEN||19700906000000|M
    O|1|6940685776||GRPH|N|20130828000000|||||||||CENTBLOOD|||||||20130828151021|||F
    R|1|ABO|A|||||F||Automatique^User||20130828151021|J271408
    M|1|A1-Cells|Reverse diluent^1^112420^19510^20140427235959^66112420_001.bmp|A1 Cells^0531^20130917235959|0^A
    M|2|B-Cells|Reverse diluent^2^112420^19510^20140427235959^66112420_001.bmp|B Cells^0531^20130917235959|30^A
    M|3|Anti-A|ABO-Rh^1^059940^27310^20140224235959^44059940_001.bmp||40^A
    M|4|Anti-B|ABO-Rh^2^059940^27310^20140224235959^44059940_001.bmp||0^A
    M|5|Anti-A+B|ABO-Rh^3^059940^27310^20140224235959^44059940_001.bmp||40^A
    M|6|Ctrl|ABO-Rh^6^059940^27310^20140224235959^44059940_001.bmp||0^A
    R|2|Rh|POS|||||F||Automatique^User||20130828151021|J271408
    M|1|Anti-D|ABO-Rh^4^059940^27310^20140224235959^44059940_001.bmp||40^A
    M|2|Anti-CDE|ABO-Rh^5^059940^27310^20140224235959^44059940_001.bmp||40^A
    M|3|Ctrl|ABO-Rh^6^059940^27310^20140224235959^44059940_001.bmp||0^A
    R|3|Pheno|Ccee|||||F||Automatique||20130828151042|J271408
    M|1|Anti-C|Rh/K^1^137572^34910^20140228235959^77137572_001.bmp||40^A
    M|2|Anti-E|Rh/K^2^137572^34910^20140228235959^77137572_001.bmp||0^A
    M|3|Anti-c|Rh/K^3^137572^34910^20140228235959^77137572_001.bmp||40^A
    M|4|Anti-e|Rh/K^4^137572^34910^20140228235959^77137572_001.bmp||40^A
    M|5|Ctrl|Rh/K^6^137572^34910^20140228235959^77137572_001.bmp||0^A
    R|4|Kell|-|||||F||Automatique||20130828151042|J271408
    M|1|Anti-K|Rh/K^5^137572^34910^20140228235959^77137572_001.bmp||0^A
    M|2|Ctrl|Rh/K^6^137572^34910^20140228235959^77137572_001.bmp||0^A
    L"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "6940685776",
                "type": "patient",
                "results": [
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "A",
                        "code": "ABO",
                        "flags": [],
                        "lots": [
                            {
                                "name": "A1 Cells_0531",
                                "lot": "A1 Cells_0531",
                                "expiration": datetime.datetime(
                                    2013, 9, 17, 23, 59, 59
                                ),
                                "type": "reagent",
                            },
                            {
                                "name": "B Cells_0531",
                                "lot": "B Cells_0531",
                                "expiration": datetime.datetime(
                                    2013, 9, 17, 23, 59, 59
                                ),
                                "type": "reagent",
                            },
                        ],
                        "operator": "User",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "Reverse diluent^1^112420^19510^2014-04-27 23:59:59^66112420_001.bmp^0",
                        "code": "ABO_A1-Cells",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "Reverse diluent^2^112420^19510^2014-04-27 23:59:59^66112420_001.bmp^30",
                        "code": "ABO_B-Cells",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "ABO-Rh^1^059940^27310^2014-02-24 23:59:59^44059940_001.bmp^40",
                        "code": "ABO_Anti-A",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "ABO-Rh^2^059940^27310^2014-02-24 23:59:59^44059940_001.bmp^0",
                        "code": "ABO_Anti-B",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "ABO-Rh^3^059940^27310^2014-02-24 23:59:59^44059940_001.bmp^40",
                        "code": "ABO_Anti-A+B",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "ABO-Rh^6^059940^27310^2014-02-24 23:59:59^44059940_001.bmp^0",
                        "code": "ABO_Ctrl",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "POS",
                        "code": "Rh",
                        "operator": "User",
                        "flags": [],
                        "lots": [],
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "ABO-Rh^4^059940^27310^2014-02-24 23:59:59^44059940_001.bmp^40",
                        "code": "Rh_Anti-D_4",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "ABO-Rh^5^059940^27310^2014-02-24 23:59:59^44059940_001.bmp^40",
                        "code": "Rh_Anti-CDE",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 21),
                        "value": "ABO-Rh^6^059940^27310^2014-02-24 23:59:59^44059940_001.bmp^0",
                        "code": "Rh_Ctrl",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Ccee",
                        "code": "Pheno",
                        "operator": "Automatique",
                        "flags": [],
                        "lots": [],
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "C",
                        "code": "Pheno_1",
                        "operator": "Automatique",
                        "flags": [],
                        "lots": [],
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "c",
                        "code": "Pheno_2",
                        "operator": "Automatique",
                        "flags": [],
                        "lots": [],
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "e",
                        "code": "Pheno_3",
                        "operator": "Automatique",
                        "flags": [],
                        "lots": [],
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "e",
                        "code": "Pheno_4",
                        "operator": "Automatique",
                        "flags": [],
                        "lots": [],
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Rh/K^1^137572^34910^2014-02-28 23:59:59^77137572_001.bmp^40",
                        "code": "Pheno_Anti-C",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Rh/K^2^137572^34910^2014-02-28 23:59:59^77137572_001.bmp^0",
                        "code": "Pheno_Anti-E",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Rh/K^3^137572^34910^2014-02-28 23:59:59^77137572_001.bmp^40",
                        "code": "Pheno_Anti-c",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Rh/K^4^137572^34910^2014-02-28 23:59:59^77137572_001.bmp^40",
                        "code": "Pheno_Anti-e",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Rh/K^6^137572^34910^2014-02-28 23:59:59^77137572_001.bmp^0",
                        "code": "Pheno_Ctrl",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "-",
                        "code": "Kell",
                        "operator": "Automatique",
                        "flags": [],
                        "lots": [],
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Rh/K^5^137572^34910^2014-02-28 23:59:59^77137572_001.bmp^0",
                        "code": "Kell_Anti-K",
                        "instrument": "J271408",
                    },
                    {
                        "date": datetime.datetime(2013, 8, 28, 15, 10, 42),
                        "value": "Rh/K^6^137572^34910^2014-02-28 23:59:59^77137572_001.bmp^0",
                        "code": "Kell_Ctrl",
                        "instrument": "J271408",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_result_vision(self):
    self.configure(traite_code_operator=True)
    result = self.decode(
        """
    H|\\^&|||OCD^VISION^1.0.4.42258^J 60002076|||||||P|LIS2-A|20150312112301
    P|1|03031279524||03031279524|ESSAI^Combs||19980101000000|M
    O|1|500000880858||COOMBS|N|20150312111137|||||||||CENTBLOOD|||||||20150312112301|||F
    R|1|IgG|POS|||S||F||Automatic||20150312112301|J 60002076
    M|1|IgG|DAT^1^019709^24110^20150924235959^24091530019709241106_20150312_112254_BW.jpg^24091530019709241106_20150312_112254_Color.jpg|IGG^0291^20161129235959|40^A
    M|2|Ctrl|DAT^3^019709^24110^20150924235959^24091530019709241106_20150312_112254_BW.jpg^24091530019709241106_20150312_112254_Color.jpg||0^A
    L"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "500000880858",
                "type": "patient",
                "results": [
                    {
                        "date": datetime.datetime(2015, 3, 12, 11, 23, 1),
                        "value": "POS",
                        "code": "IgG",
                        "flags": ["Automatic"],
                        "lots": [
                            {
                                "name": "IGG_0291",
                                "expiration": datetime.datetime(
                                    2016, 11, 29, 23, 59, 59
                                ),
                                "lot": "IGG_0291",
                                "type": "reagent",
                            }
                        ],
                        "operator": "Automatic",
                        "instrument": "J 60002076",
                    },
                    {
                        "date": datetime.datetime(2015, 3, 12, 11, 23, 1),
                        "value": "DAT^1^019709^24110^2015-09-24 23:59:59^24091530019709241106_20150312_112254_Color.jpg^40",
                        "code": "IgG_IgG",
                        "instrument": "J 60002076",
                    },
                    {
                        "date": datetime.datetime(2015, 3, 12, 11, 23, 1),
                        "value": "DAT^3^019709^24110^2015-09-24 23:59:59^24091530019709241106_20150312_112254_Color.jpg^0",
                        "code": "IgG_Ctrl",
                        "instrument": "J 60002076",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_result_vision_with_alarms(self):
    self.configure(recuperer_alarmes=True)
    result = self.decode(
        """
    H|\\^&|||OCD^VISION^1.0.4.42258^J 60002076|||||||P|LIS2-A|20150312112301
    P|1|03031279524||03031279524|ESSAI^Combs||19980101000000|M
    O|1|500000880858||COOMBS|N|20150312111137|||||||||CENTBLOOD|||||||20150312112301|||F
    R|1|IgG|POS|||S||F||Automatic||20150312112301|J 60002076
    M|1|IgG|DAT^1^019709^24110^20150924235959^24091530019709241106_20150312_112254_BW.jpg^24091530019709241106_20150312_112254_Color.jpg||40^A
    M|2|Ctrl|DAT^3^019709^24110^20150924235959^24091530019709241106_20150312_112254_BW.jpg^24091530019709241106_20150312_112254_Color.jpg||0^A
    L"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "500000880858",
                "type": "patient",
                "results": [
                    {
                        "date": datetime.datetime(2015, 3, 12, 11, 23, 1),
                        "value": "POS",
                        "code": "IgG",
                        "flags": ["0", "40", "S"],
                        "operator": "Automatic",
                        "instrument": "J 60002076",
                        "lots": [],
                    },
                    {
                        "date": datetime.datetime(2015, 3, 12, 11, 23, 1),
                        "value": "DAT^1^019709^24110^2015-09-24 23:59:59^24091530019709241106_20150312_112254_Color.jpg^40",
                        "code": "IgG_IgG",
                        "instrument": "J 60002076",
                    },
                    {
                        "date": datetime.datetime(2015, 3, 12, 11, 23, 1),
                        "value": "DAT^3^019709^24110^2015-09-24 23:59:59^24091530019709241106_20150312_112254_Color.jpg^0",
                        "code": "IgG_Ctrl",
                        "instrument": "J 60002076",
                    },
                ],
            }
        ],
    }
    assert result == expected
