# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.tests import BaseTest

parser_class = "cd3200"


def test_cbc_resultat(self):
    date = datetime.datetime(1996, 5, 24, 19, 25)
    result = self.decode(
        """"   ","CD3200C","------------",0416,0,0,"123456789   ","----------------------------","----------------","M","10/25/1975","----------------------","stc","05/24/1996","19:25","05/24/1996","11:48","----------------",05.00,03.20,01.20,0.300,0.200,0.100,04.94,015.3,043.1,092.3,031.0,034.4,012.0,00177,010.7,01.90,019.2,064.0,024.0,06.00,04.00,02.00,1,1,0,0,1,1,0,0,0,0,0,0,0,1,0,0,0,1,0,0,0,0,1,0,00000,00000,00000,00000,00000,00000,00000,00000,-----,05.00,3,1,0,1,1,1,0,"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "WBC", "comments": [], "date": date, "value": "05.00"},
                    {"code": "NEU", "comments": [], "date": date, "value": "03.20"},
                    {"code": "LYM", "comments": [], "date": date, "value": "01.20"},
                    {"code": "MONO", "comments": [], "date": date, "value": "0.300"},
                    {"code": "EOS", "comments": [], "date": date, "value": "0.200"},
                    {"code": "BASO", "comments": [], "date": date, "value": "0.100"},
                    {"code": "RBC", "comments": [], "date": date, "value": "04.94"},
                    {"code": "HGB", "comments": [], "date": date, "value": "015.3"},
                    {"code": "HCT", "comments": [], "date": date, "value": "043.1"},
                    {"code": "MCV", "comments": [], "date": date, "value": "092.3"},
                    {"code": "MCH", "comments": [], "date": date, "value": "031.0"},
                    {"code": "MCHC", "comments": [], "date": date, "value": "034.4"},
                    {"code": "RDW", "comments": [], "date": date, "value": "012.0"},
                    {"code": "PLT", "comments": [], "date": date, "value": "00177"},
                    {"code": "MPV", "comments": [], "date": date, "value": "010.7"},
                    {"code": "PCT", "comments": [], "date": date, "value": "01.90"},
                    {"code": "PDW", "comments": [], "date": date, "value": "019.2"},
                    {"code": "NEU%", "comments": [], "date": date, "value": "064.0"},
                    {"code": "LYM%", "comments": [], "date": date, "value": "024.0"},
                    {"code": "MONO%", "comments": [], "date": date, "value": "06.00"},
                    {"code": "EOS%", "comments": [], "date": date, "value": "04.00"},
                    {"code": "BASO%", "comments": [], "date": date, "value": "02.00"},
                    {
                        "code": "ALARME",
                        "comments": [
                            "Moving Average",
                            "DFLT",
                            "DFLT Neutrophil Qualifier",
                            "DFLT Eosinophil Qualifier",
                            "WBC",
                            "RBC MORPH",
                            "LRI",
                        ],
                        "date": date,
                        "value": "",
                    },
                ],
                "sample_id": "123456789",
                "type": "patient",
            }
        ],
    }
    assert result == expected


def test_retic_resultats(self):
    date = datetime.datetime(1995, 1, 25, 14, 8)
    result = self.decode(
        """"RTC","CD3200C","------------",0416,0,0,"123456789   ","----------------------------","----------------","M","10/25/1975","----------------------","stc","01/25/1995","14:08","01/24/1995","12:08","----------------","1.000","50.00","5.000","0.000","-----","0.000","0.000","0.000","0.000","0.000",0,0,0,0,0,0,0,0,0,0,"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {"code": "RETIC%", "comments": [], "date": date, "value": "1.000"},
                    {
                        "code": "RETIC_ABS",
                        "comments": [],
                        "date": date,
                        "value": "50.00",
                    },
                    {"code": "RBC", "comments": [], "date": date, "value": "5.000"},
                    {
                        "code": "BACKGROUND",
                        "comments": [],
                        "date": date,
                        "value": "0.000",
                    },
                    {
                        "code": "IMMATURE_RETIC_FRACTION",
                        "comments": [],
                        "date": date,
                        "value": "",
                    },
                ],
                "sample_id": "123456789",
                "type": "patient",
            }
        ],
    }
    assert result == expected


def test_cbc_qc_resultats(self):
    date = datetime.datetime(1995, 1, 25, 14, 8)
    result = self.decode(
        """"RTC","CD3200C","------------",0416,0,1,"123456789   ","----------------------------","----------------","M","10/25/1975","----------------------","stc","01/25/1995","14:08","01/24/1995","12:08","----------------","1.000","50.00","5.000","0.000","-----","0.000","0.000","0.000","0.000","0.000",0,0,0,0,0,0,0,0,0,0,"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "123456789",
                "results": [
                    {"code": "RETIC%", "date": date, "value": "1.000"},
                    {"code": "RETIC_ABS", "date": date, "value": "50.00"},
                    {"code": "RBC", "date": date, "value": "5.000"},
                    {"code": "BACKGROUND", "date": date, "value": "0.000"},
                    {"code": "IMMATURE_RETIC_FRACTION", "date": date, "value": ""},
                ],
                "sample_id": "123456789",
                "type": "control",
            }
        ],
    }
    assert result == expected


def test_order(self):
    message = self.order()
    expected = '"123456      ","NYME ANO                    ","036666666       ","F","05/13/2011","05/13/2013","11:42","PRESCRIBER            ",0,0,'
    assert message == expected


class Cd3200Test(BaseTest):
    PROTOCOL_CLASS = "cd3200"

    def test_send_recv(self):
        self.assertSendRecvMessageEqual(
            '"123456      ","NYME ANO                    ","036666666       ","F","05/13/2011","05/13/2013","11:42","PRS                   ",0,0,',
            """
            send <stx>"123456      ","NYME ANO                    ","036666666       ","F","05/13/2011","05/13/2013","11:42","PRS                   ",0,0,16<etx>
            recv <ack>
            """,
        )
