# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "biorad_d10"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||||20130715141912
    P|1
    O|1|123456||^^^A
    O|2|123456||^^^B
    L|1|N
    """
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||Bio-Rad CDM System^CDM 5.1 VII TURBO Instrument 11817|||||||||20140717124720
    P|1
    O|1|600717007255||^^^4||||||||||||||||||||1
    R|1|^^^A1a^AREA|0.7|||||||||20140227160959
    R|2|^^^A1a^TIME|0.161|||||||||20140227160959
    R|3|^^^A1b^AREA|1.4|||||||||20140227160959
    R|4|^^^A1b^TIME|0.232|||||||||20140227160959
    R|5|^^^F^AREA|0.8|||||||||20140227160959
    R|6|^^^F^TIME|0.293|||||||||20140227160959
    R|7|^^^LA1c^AREA|2.2|||||||||20140227160959
    R|8|^^^LA1c^TIME|0.412|||||||||20140227160959
    R|9|^^^A1c^AREA|7.1|||||||||20140227160959
    R|10|^^^A1cIFCC^AREA|54|||||||||20140227160959
    R|11|^^^A1c^TIME|0.517|||||||||20140227160959
    R|12|^^^P3^AREA|3.9|||||||||20140227160959
    R|13|^^^P3^TIME|0.769|||||||||20140227160959
    R|14|^^^P4^AREA|1.4|||||||||20140227160959
    R|15|^^^P4^TIME|0.865|||||||||20140227160959
    R|16|^^^Ao^AREA|83.1|||||||||20140227160959
    R|17|^^^Ao^TIME|1.019|||||||||20140227160959
    R|18|^^^TOTAL^AREA|1.39|||||||||20140227160959
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "600717007255",
                "results": [
                    {
                        "code": "A1aAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.7",
                    },
                    {
                        "code": "A1aTIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.161",
                    },
                    {
                        "code": "A1bAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "1.4",
                    },
                    {
                        "code": "A1bTIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.232",
                    },
                    {
                        "code": "FAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.8",
                    },
                    {
                        "code": "FTIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.293",
                    },
                    {
                        "code": "LA1cAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "2.2",
                    },
                    {
                        "code": "LA1cTIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.412",
                    },
                    {
                        "code": "A1cAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "7.1",
                    },
                    {
                        "code": "A1cIFCCAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "54",
                    },
                    {
                        "code": "A1cTIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.517",
                    },
                    {
                        "code": "P3AREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "3.9",
                    },
                    {
                        "code": "P3TIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.769",
                    },
                    {
                        "code": "P4AREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "1.4",
                    },
                    {
                        "code": "P4TIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "0.865",
                    },
                    {
                        "code": "AoAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "83.1",
                    },
                    {
                        "code": "AoTIME",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "1.019",
                    },
                    {
                        "code": "TOTALAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "1.39",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||Bio-Rad CDM System^CDM 5.1 VII TURBO Instrument 11817|||||||||20140717124720
    Q|1|^12345||^^^4
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "12345"}],
    }
    assert worklist == expected
