# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

import driverng

parser_class = "dms_tracking"


def test_decode_result(self):
    self.configure(nom_chaine_dms="DMS-A")
    worklist = self.decode(
        """
    H|\\^&||111040||Genoa 1|||||||P|1
    M|1|WARN|1272249735|20150305105913|IOM2|L01|Tube Not Detected
    M|2|STOR|1077992693|20150305105913|IOM|006726^00^04^22
    M|3|SAMP|1077010008|20150305105914|ADV2120-1
    M|4|CENT|1002117064|20150305105914
    M|5|DECA|1008513467|20150305105915
    M|6|QRY|1077008990|20150305105918|ADV2120-1|006\\009\\011\\024\\025\\008\\003\\022\\021
    M|7|CKIN|1272256981|20150305105901|RIM/1Siero|6
    M|8|INSTRUMENTEXCEPTION|1272256880|20150305110157|CENTAUR1|H|DMS: ||TSHR
    M|9|INSTRUMENTEXCEPTION|1002117217|20150305110736|ADV1800-1|H|anormale H||PCR
    M|1|ALIQINFO|1030055701|20170330162632|1030055701||500|SEROT|SEROTH|ALIQUOTER
    M|2|SSU|ULRICH|L^Left Equipment^HL70370|04-ALci-OR^SCM01690^''^''^M1896^1|20181219163423
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "1272249735",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["Tube Not Detected"],
                        "date": datetime.datetime(2015, 3, 5, 10, 59, 13),
                        "flags": ["L01"],
                        "instrument": "DMS-A_IOM2",
                        "value": "WARN",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1077992693",
                "rack": "IOM-006726-00-04-22",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["IOM-006726-00-04-22"],
                        "date": datetime.datetime(2015, 3, 5, 10, 59, 13),
                        "flags": [],
                        "instrument": "DMS-A_IOM",
                        "value": "STOR",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1077010008",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": datetime.datetime(2015, 3, 5, 10, 59, 14),
                        "flags": [],
                        "instrument": "DMS-A_ADV2120-1",
                        "value": "SAMP",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1002117064",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": datetime.datetime(2015, 3, 5, 10, 59, 14),
                        "flags": [],
                        "instrument": "",
                        "value": "CENT",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1008513467",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": datetime.datetime(2015, 3, 5, 10, 59, 15),
                        "flags": [],
                        "instrument": "",
                        "value": "DECA",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1077008990",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [
                            "006",
                            "009",
                            "011",
                            "024",
                            "025",
                            "008",
                            "003",
                            "022",
                            "021",
                        ],
                        "date": datetime.datetime(2015, 3, 5, 10, 59, 18),
                        "flags": [],
                        "instrument": "DMS-A_ADV2120-1",
                        "value": "QRY",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1272256981",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": datetime.datetime(2015, 3, 5, 10, 59, 1),
                        "flags": ["6"],
                        "instrument": "DMS-A_RIM/1Siero",
                        "value": "CKIN",
                        "date_plateau": datetime.datetime(2015, 3, 5, 10, 59, 1),
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1272256880",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["DMS:  TSHR"],
                        "date": datetime.datetime(2015, 3, 5, 11, 1, 57),
                        "flags": ["H"],
                        "instrument": "DMS-A_CENTAUR1",
                        "value": "INSTRUMENTEXCEPTION",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1002117217",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["anormale H PCR"],
                        "date": datetime.datetime(2015, 3, 5, 11, 7, 36),
                        "flags": ["H"],
                        "instrument": "DMS-A_ADV1800-1",
                        "value": "INSTRUMENTEXCEPTION",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1030055701",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["1030055701_SEROTH"],
                        "date": datetime.datetime(2017, 3, 30, 16, 26, 32),
                        "flags": ["500"],
                        "instrument": "DMS-A_ALIQUOTER",
                        "value": "ALIQINFO",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "ULRICH",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["L-Left Equipment-HL70370"],
                        "date": datetime.datetime(2018, 12, 19, 16, 34, 23),
                        "flags": [],
                        "instrument": "DMS-A_04-ALci-OR",
                        "value": "SSU",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
        ],
    }
    assert worklist == expected


def test_decode_tracking(self):
    worklist = self.decode(
        """
    H|\\^&||141255||DMS|||||||P|1
    M|1|SSU|690126026222|L^Left Equipment^HL70370|04-ALci-OR^SCM01690^''^''^M2396^1|20190226140818
    M|2|ECK|580126029596|LIP|OK^Notification or request accepted^HL70119|SC^In process, scheduled^HL70038|20190226141233
    M|3|ECK|580126029596|PAL|OK^Notification or request accepted^HL70119|SC^In process, scheduled^HL70038|20190226141233
    M|4|ECK|580126029596|TRIGLY|OK^Notification or request accepted^HL70119|SC^In process, scheduled^HL70038|20190226141233
    M|5|TSC|761226027498|GLU|INITIATED^Initiated^99ABT|20190226141218|20190226141234
    M|6|TSC|750226024955|UR-U|INITIATED^Initiated^99ABT|20190226141618|20190226141235
    M|7|CKIN|680126027933|20190226141236|RIM|42|
    M|8|SORTINFO|S0C413290019|STOR1|20201124215954|SORTALT
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "690126026222",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["L-Left Equipment-HL70370"],
                        "date": datetime.datetime(2019, 2, 26, 14, 8, 18),
                        "flags": [],
                        "instrument": "DMS_04-ALci-OR",
                        "value": "SSU",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "680126027933",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": datetime.datetime(2019, 2, 26, 14, 12, 36),
                        "flags": ["42"],
                        "instrument": "DMS_RIM",
                        "value": "CKIN",
                        "date_plateau": datetime.datetime(2019, 2, 26, 14, 12, 36),
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "S0C413290019",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": datetime.datetime(2020, 11, 24, 21, 59, 54),
                        "flags": ["SORTALT"],
                        "instrument": "DMS_STOR1",
                        "value": "SORTINFO",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
        ],
    }
    assert worklist == expected


def test_encode_enable_disable(self):
    worklist = driverng.Worklist(
        context="order",
        samples=[
            {
                "sample_id": "ADV1800-1",
                "type": "patient",
                "orders": [
                    {"code": "CK", "action": "add"},
                    {"code": "NA", "action": "del"},
                ],
            },
            {
                "sample_id": "ADV1800-2",
                "type": "patient",
                "orders": [{"code": "*", "action": "del"}],
            },
        ],
    )
    message = self.encode(worklist)
    assert (
        message
        == """
    H|\\^&||||||||||P|1|20130715141912
    M|1|ENABLE|ADV1800-1|CK
    M|2|DISABLE|ADV1800-1|NA
    M|3|DISABLE|ADV1800-2|*
    L|1"""
    )


def test_decode_result_M(self):
    worklist = self.decode(
        """
        H|\\^&||||||||||P|1|20130715141912
        M|1|ENABLE|ADV1800-1|CK
        L|1"""
    )
    assert not worklist.samples


def test_decode_result_REQ(self):
    self.configure(nom_chaine_dms="DMS-REQ")
    worklist = self.decode(
        """
    H|\\^&||072023||DMS|||||||P|1
    M|1|STOR|491220110033|20220111072001|IOM|151488^00^06^16
    M|2|REQ^DELIVERY|221020110030|20220111072001|ref
    M|3|REQ^ERROR|071020111023|20220111103229|Nemo
    M|4|CKIN|F220087096PA|20220111103550|RIM-3|42|0
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "491220110033",
                "rack": "IOM-151488-00-06-16",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["IOM-151488-00-06-16"],
                        "date": datetime.datetime(2022, 1, 11, 7, 20, 1),
                        "flags": [],
                        "instrument": "DMS-REQ_IOM",
                        "value": "STOR",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "221020110030",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["DELIVERY"],
                        "date": datetime.datetime(2022, 1, 11, 7, 20, 1),
                        "flags": [],
                        "instrument": "",
                        "value": "REQ",
                        "date_plateau": None,
                        "operator": "ref",
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "071020111023",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["ERROR"],
                        "date": datetime.datetime(2022, 1, 11, 10, 32, 29),
                        "flags": [],
                        "instrument": "",
                        "value": "REQ",
                        "date_plateau": None,
                        "operator": "Nemo",
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "F220087096PA",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["0"],
                        "date": datetime.datetime(2022, 1, 11, 10, 35, 50),
                        "flags": ["42"],
                        "instrument": "DMS-REQ_RIM-3",
                        "value": "CKIN",
                        "date_plateau": datetime.datetime(2022, 1, 11, 10, 35, 50),
                        "operator": None,
                    }
                ],
            },
        ],
    }
    assert worklist == expected


def test_decode_result_Divers(self):
    self.configure(nom_chaine_dms="DMS-DIV")
    worklist = self.decode(
        """
    H|\\^&||072023||DMS|||||||P|1
    M|1|ROUT|ROUT_sampleID|20220111103550|analyzerID|Test-1\\Test‐Rout
    M|1|SORTINFO|SORTINFO_SID|instrument|dateTime|Test1\\TestSortInfo
    M|1|ALIQINFO|1030055701|20170330162632|1030055701||500|SEROT|SEROTH\\Test_AliqInfo|ALIQUOTER
    M|1|PLATEINFO|50076428|400015|1021221856^A3^100\\UNKNOWN^B1^100\\1021221895^D2^100\\1021221896^F3^100\
1021221867^G1^100|20151201125148|MLHR_1
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "ROUT_sampleID",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": datetime.datetime(2022, 1, 11, 10, 35, 50),
                        "flags": ["Test-1", "Test‐Rout"],
                        "instrument": "DMS-DIV_analyzerID",
                        "value": "ROUT",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "SORTINFO_SID",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": [],
                        "date": None,
                        "flags": ["Test1", "TestSortInfo"],
                        "instrument": "DMS-DIV_instrument",
                        "value": "SORTINFO",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "1030055701",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["1030055701_SEROTH-Test_AliqInfo"],
                        "date": datetime.datetime(2017, 3, 30, 16, 26, 32),
                        "flags": ["500"],
                        "instrument": "DMS-DIV_ALIQUOTER",
                        "value": "ALIQINFO",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "400015",
                "results": [
                    {
                        "code": "TRACKING",
                        "comments": ["50076428"],
                        "date": datetime.datetime(2015, 12, 1, 12, 51, 48),
                        "flags": [],
                        "instrument": "DMS-DIV_MLHR_1",
                        "value": "PLATEINFO",
                        "date_plateau": None,
                        "operator": None,
                    }
                ],
            },
        ],
    }
    assert worklist == expected
