# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy

parser_class = "dxc700"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].specimen = "W"
    message = self.order(
        worklist=worklist,
        context={"query": self.parse("R 99123412W1234                    123456")},
    )
    expected = "S 123412W1234          123456    EF001  036666666           NYME ANO            A  0B  0"
    assert message == expected


def test_decode_qc(self):
    worklist = self.decode(
        "DQ02      UQ001              U2 052E0352663C9562663D834    20.49        0272696B3522696B062     3.55        0322584152325841441   494.67        0312678M178           108.53        1132526M3052526M261  1188.26        1112647150826471351      201        0592203258022030749   152.65        11026720390             0.88        02225707179           159.31        "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "U2",
                "results": [
                    {
                        "code": "035",
                        "flags": [],
                        "lots": [
                            {"lot": "2663", "type": "reagent"},
                            {"lot": "C956", "type": "reagent"},
                            {"lot": "2663", "type": "reagent"},
                            {"lot": "D834", "type": "reagent"},
                        ],
                        "value": "20.49",
                    },
                    {
                        "code": "027",
                        "flags": [],
                        "lots": [
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B352", "type": "reagent"},
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B062", "type": "reagent"},
                        ],
                        "value": "3.55",
                    },
                    {
                        "code": "032",
                        "flags": [],
                        "lots": [
                            {"lot": "2584", "type": "reagent"},
                            {"lot": "1523", "type": "reagent"},
                            {"lot": "2584", "type": "reagent"},
                            {"lot": "1441", "type": "reagent"},
                        ],
                        "value": "494.67",
                    },
                    {
                        "code": "031",
                        "flags": [],
                        "lots": [
                            {"lot": "2678", "type": "reagent"},
                            {"lot": "M178", "type": "reagent"},
                        ],
                        "value": "108.53",
                    },
                    {
                        "code": "113",
                        "flags": [],
                        "lots": [
                            {"lot": "2526", "type": "reagent"},
                            {"lot": "M305", "type": "reagent"},
                            {"lot": "2526", "type": "reagent"},
                            {"lot": "M261", "type": "reagent"},
                        ],
                        "value": "1188.26",
                    },
                    {
                        "code": "111",
                        "flags": [],
                        "lots": [
                            {"lot": "2647", "type": "reagent"},
                            {"lot": "1508", "type": "reagent"},
                            {"lot": "2647", "type": "reagent"},
                            {"lot": "1351", "type": "reagent"},
                        ],
                        "value": "201",
                    },
                    {
                        "code": "059",
                        "flags": [],
                        "lots": [
                            {"lot": "2203", "type": "reagent"},
                            {"lot": "2580", "type": "reagent"},
                            {"lot": "2203", "type": "reagent"},
                            {"lot": "0749", "type": "reagent"},
                        ],
                        "value": "152.65",
                    },
                    {
                        "code": "110",
                        "flags": [],
                        "lots": [
                            {"lot": "2672", "type": "reagent"},
                            {"lot": "0390", "type": "reagent"},
                        ],
                        "value": "0.88",
                    },
                    {
                        "code": "022",
                        "flags": [],
                        "lots": [
                            {"lot": "2570", "type": "reagent"},
                            {"lot": "7179", "type": "reagent"},
                        ],
                        "value": "159.31",
                    },
                ],
                "sample_id": "U2",
                "specimen": "U",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_iono_qc(self):
    worklist = self.decode(
        "DQ02       Q001              L2 001E097    133.2        098     3.87        099    100.1        "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "L2",
                "results": [
                    {"code": "097", "flags": [], "lots": [], "value": "133.2"},
                    {"code": "098", "flags": [], "lots": [], "value": "3.87"},
                    {"code": "099", "flags": [], "lots": [], "value": "100.1"},
                ],
                "sample_id": "L2",
                "specimen": "S",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        "D 02001002 0002    150000016001    EM0760049068               BREMENT Alain       0272696B3522696B062     0.89        02326244421             2.19        0282476831024767748     0.52        0342714268727142611     1.31        0492737D1772737D130     2.35        09626173722          0  0  0        "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0010-02",
                "results": [
                    {
                        "code": "027",
                        "flags": [],
                        "lots": [
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B352", "type": "reagent"},
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B062", "type": "reagent"},
                        ],
                        "value": "0.89",
                    },
                    {
                        "code": "023",
                        "flags": [],
                        "lots": [
                            {"lot": "2624", "type": "reagent"},
                            {"lot": "4421", "type": "reagent"},
                        ],
                        "value": "2.19",
                    },
                    {
                        "code": "028",
                        "flags": [],
                        "lots": [
                            {"lot": "2476", "type": "reagent"},
                            {"lot": "8310", "type": "reagent"},
                            {"lot": "2476", "type": "reagent"},
                            {"lot": "7748", "type": "reagent"},
                        ],
                        "value": "0.52",
                    },
                    {
                        "code": "034",
                        "flags": [],
                        "lots": [
                            {"lot": "2714", "type": "reagent"},
                            {"lot": "2687", "type": "reagent"},
                            {"lot": "2714", "type": "reagent"},
                            {"lot": "2611", "type": "reagent"},
                        ],
                        "value": "1.31",
                    },
                    {
                        "code": "049",
                        "flags": [],
                        "lots": [
                            {"lot": "2737", "type": "reagent"},
                            {"lot": "D177", "type": "reagent"},
                            {"lot": "2737", "type": "reagent"},
                            {"lot": "D130", "type": "reagent"},
                        ],
                        "value": "2.35",
                    },
                    {
                        "code": "096",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0  0  0",
                    },
                    {
                        "code": "L",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "I",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "H",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                ],
                "sample_id": "150000016001",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_iono_result(self):
    worklist = self.decode(
        "D 02000101 0001    150000006301    EF0020048870               Z_LABO Dï¿œborah    097    152.5r       098     0.77G r     "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0001-01",
                "results": [
                    {"code": "097", "flags": ["r"], "lots": [], "value": "152.5"},
                    {"code": "098", "flags": ["G", "r"], "lots": [], "value": "0.77"},
                ],
                "sample_id": "150000006301",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_hil_result(self):
    worklist = self.decode(
        "D 02001001 0013    150000015501    EF0020048879               Z_LABO Dï¿œborah    0272696B3522696B062     0.98        09626173722          0  0  0        "
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "0010-01",
                "results": [
                    {
                        "code": "027",
                        "flags": [],
                        "lots": [
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B352", "type": "reagent"},
                            {"lot": "2696", "type": "reagent"},
                            {"lot": "B062", "type": "reagent"},
                        ],
                        "value": "0.98",
                    },
                    {
                        "code": "096",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0  0  0",
                    },
                    {
                        "code": "L",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "I",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                    {
                        "code": "H",
                        "flags": [],
                        "lots": [
                            {"lot": "2617", "type": "reagent"},
                            {"lot": "3722", "type": "reagent"},
                        ],
                        "value": "0",
                    },
                ],
                "sample_id": "150000015501",
                "specimen": "S",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
