# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "evolis"


def test_order(self):
    result = self.order()
    expected = """H|\\^&||||||||||||20130715141912
        P|1|036666666|123456||NYME^ANO||20110513|F|||||PRESCRIBER
        O|1|123456||^^^A\\^^^B|||20130513114242||||||||Serum||||||||||X
        L|1|N"""
    assert result == expected


def test_encode_order_without_prescriber(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0]["physicians"] = []
    result = self.order(worklist=worklist)
    expected = """H|\\^&||||||||||||20130715141912
        P|1|036666666|123456||NYME^ANO||20110513|F
        O|1|123456||^^^A\\^^^B|||20130513114242||||||||Serum||||||||||X
        L|1|N"""
    assert result == expected


def test_result(self):
    date = datetime.datetime(2014, 2, 18, 20, 27, 38)
    result = self.decode(
        """H|\\^&|||LaborEDV||||||||1|20140218202738
        P|1|PatID01|42670070||Meir^Anna||19741001|F|||||MARTINEZ
        O|1|||^^^TOXM||20011115|||||||||S||||||||||X
        R|1|^^^TOXOM^TOXOM|NEG
        C|1|I|Toxo IgM^3L0024^30/10/2014|I
        R|2|^^^TOXOM^ITOXOM|0.2212
        L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "42670070",
                "type": "patient",
                "results": [
                    {
                        "flags": [],
                        "date": date,
                        "lots": [
                            {
                                "name": "Toxo IgM",
                                "lot": "3L0024",
                                "expiration": datetime.datetime(2014, 10, 30),
                                "type": "reagent",
                            }
                        ],
                        "value": "NEG",
                        "code": "TOXOM",
                    },
                    {
                        "flags": [],
                        "date": date,
                        "lots": [],
                        "value": "0.2212",
                        "code": "ITOXOM",
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_control(self):
    self.configure(evolis_reference_qc="NC;PC;CO;CN")
    date = datetime.datetime(2019, 2, 18, 20, 27, 38)
    result = self.decode(
        """
    H|\\^&|||LaborEDV||||||||1|20190218202738
    P|1
    O|1||NC1|^^^ASPAG
    R|1|^^^ASPAG^ASPAG_QUANT|0.1396||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    R|2|^^^ASPAG^ASPAG_QUAL|NC1||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    P|2
    O|1||PC1|^^^ASPAG
    R|1|^^^ASPAG^ASPAG_QUANT|2.2569||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    R|2|^^^ASPAG^ASPAG_QUAL|PC1||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    P|3
    O|1||CO1|^^^ASPAG
    R|1|^^^ASPAG^ASPAG_QUANT|1.0230||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    R|2|^^^ASPAG^ASPAG_QUAL|CO1||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    P|4
    O|1||CO2|^^^ASPAG
    R|1|^^^ASPAG^ASPAG_QUANT|0.9770||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    R|2|^^^ASPAG^ASPAG_QUAL|CO2||
    C|1|I|Aspergillus Ag ETP BR V20^9E0063^25/3/20|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "NC1",
                "control_lot": "NC1",
                "type": "control",
                "results": [
                    {
                        "code": "ASPAG_QUANT",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "0.1396",
                    },
                    {
                        "code": "ASPAG_QUAL",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "NC1",
                    },
                ],
            },
            {
                "sample_id": "PC1",
                "control_lot": "PC1",
                "type": "control",
                "results": [
                    {
                        "code": "ASPAG_QUANT",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "2.2569",
                    },
                    {
                        "code": "ASPAG_QUAL",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "PC1",
                    },
                ],
            },
            {
                "sample_id": "CO1",
                "control_lot": "CO1",
                "type": "control",
                "results": [
                    {
                        "code": "ASPAG_QUANT",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "1.0230",
                    },
                    {
                        "code": "ASPAG_QUAL",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "CO1",
                    },
                ],
            },
            {
                "sample_id": "CO2",
                "control_lot": "CO2",
                "type": "control",
                "results": [
                    {
                        "code": "ASPAG_QUANT",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "0.9770",
                    },
                    {
                        "code": "ASPAG_QUAL",
                        "date": date,
                        "flags": [],
                        "lots": [
                            {
                                "expiration": None,
                                "lot": "9E0063",
                                "name": "Aspergillus Ag ETP BR V20",
                                "type": "reagent",
                            }
                        ],
                        "value": "CO2",
                    },
                ],
            },
        ],
    }
    assert result == expected
