# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "hl7_sil"


def test_decode_order1(self):
    worklist = self.decode(
        """
    MSH|^~\\&|middleware||SIL||20101216095928||OML^O33^OML_O33|MSG0000057367|P|2.5|||AL|NE|
    PID|1|4645615191|0012541258||DUPONT^Sophie|DURAND|20011212|F|||10, rue test^chemin tutu^Toulouse^^31000^France|||0102030405
    NTE|1|P|Infos patient|
    PV1|1||SURG^Chambre^lit||||DUCH||||||||||||ADM_CLI|||||||||||||||20101210080101||||||||||||
    SPM|1|01||SERUM|
    OBR|1||0100000666|SANSORC^SANSORC||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|NW||0100000667|0100000667|CM||||||||||20101216085010||
    OBR|1||0100000667|K^POTASSIUM|||||||||RensCli3|||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|NW||0100000667|0100000667|CM||||||||||20101216085010||
    OBR|1||0100000667|CL^CHLORINE||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|NW||0100000667|0100000667|CM||||||||||20101216085010||
    OBR|1||0100000667|CRE^CREATIN||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|NW||0100000667|0100000667|CM||||||||||20101216085010||
    OBR|1||0100000667|UREA^UREA||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    NTE|1|P|Obs tube|
    OBX|1|NM|NA||12|mmol/L|||||F|||20110124134900||||||
    OBR|3|1210000015|121000001501|ALBSNG|||||||||||||||||||||||^^^20210226100800^^R
    OBX|3|ST|ALBSNG||55.00|g/L||||||||20210225154000|||||20210225174126
    OBX|3|ST|ACCUEIL||Résu Accueil|g/L|||||||||||||20210225174126
    OBX|3|ST|BADDATE||Résu Accueil|g/L||||||||20210231174126|||||20210231174126
    """
    )
    expected = {
        "samples": [
            {
                "type": "patient",
                "sample_id": "0100000666",
                "request_id": "0012541258",
                "patient_id": "4645615191",
                "last_name": "DUPONT",
                "first_name": "Sophie",
                "maiden_name": "DURAND",
                "birthdate": datetime.datetime(2001, 12, 12, 0, 0),
                "sex": "F",
                "location": "SURG",
                "alternate_patient_id": "ADM_CLI",
                "admission_date": None,
                "room": "Chambre",
                "bed": "lit",
                "collection_date": datetime.datetime(2010, 12, 16, 9, 58, 12),
                "request_date": datetime.datetime(2010, 12, 16, 9, 58, 12),
                "city": "Toulouse",
                "postal_code": "31000",
                "country": "France",
                "address": "10, rue test~chemin tutu",
                "phone": "0102030405",
                "comments": [{"type": "patient", "text": "Infos patient"}],
                "orders": [{"code": "SANSORC", "priority": "R", "action": "add"}],
                "physicians": [
                    {"code": "DUCH", "last_name": "DUCHEMIN", "type": "prescriber"}
                ],
            },
            {
                "type": "patient",
                "sample_id": "0100000667",
                "request_id": "0012541258",
                "patient_id": "4645615191",
                "last_name": "DUPONT",
                "first_name": "Sophie",
                "maiden_name": "DURAND",
                "birthdate": datetime.datetime(2001, 12, 12, 0, 0),
                "sex": "F",
                "diagnostic": "RensCli3",
                "address": "10, rue test~chemin tutu",
                "postal_code": "31000",
                "city": "Toulouse",
                "country": "France",
                "phone": "0102030405",
                "location": "SURG",
                "room": "Chambre",
                "bed": "lit",
                "alternate_patient_id": "ADM_CLI",
                "admission_date": None,
                "collection_date": datetime.datetime(2010, 12, 16, 9, 58, 12),
                "request_date": datetime.datetime(2010, 12, 16, 8, 50, 10),
                "comments": [
                    {"type": "patient", "text": "Infos patient"},
                    {"type": "sample", "text": "Obs tube"},
                ],
                "orders": [
                    {"code": "K", "priority": "R", "action": "add"},
                    {"code": "CL", "priority": "R", "action": "add"},
                    {"code": "CRE", "priority": "R", "action": "add"},
                    {"code": "UREA", "priority": "R", "action": "add"},
                ],
                "physicians": [
                    {"code": "DUCH", "last_name": "DUCHEMIN", "type": "prescriber"}
                ],
                "results": [
                    {
                        "type": "NM",
                        "anterior": True,
                        "code": "NA",
                        "value": "12",
                        "date": datetime.datetime(2011, 1, 24, 13, 49),
                    }
                ],
            },
            {
                "type": "patient",
                "sample_id": "121000001501",
                "request_id": "1210000015",
                "patient_id": "4645615191",
                "last_name": "DUPONT",
                "first_name": "Sophie",
                "maiden_name": "DURAND",
                "birthdate": datetime.datetime(2001, 12, 12, 0, 0),
                "sex": "F",
                "address": "10, rue test~chemin tutu",
                "postal_code": "31000",
                "city": "Toulouse",
                "country": "France",
                "phone": "0102030405",
                "location": "SURG",
                "room": "Chambre",
                "bed": "lit",
                "location": "SURG",
                "alternate_patient_id": "ADM_CLI",
                "admission_date": None,
                "request_date": datetime.datetime(2010, 12, 16, 8, 50, 10),
                "collection_date": datetime.datetime(2021, 2, 26, 10, 8),
                "comments": [{"type": "patient", "text": "Infos patient"}],
                "orders": [
                    {"code": "ALBSNG", "priority": "R", "action": "add"},
                    {"code": "ACCUEIL", "action": "add"},
                ],
                "results": [
                    {
                        "type": "ST",
                        "anterior": True,
                        "code": "ALBSNG",
                        "value": "55.00",
                        "date": datetime.datetime(2021, 2, 25, 15, 40),
                    },
                    {
                        "type": "ST",
                        "anterior": False,
                        "code": "ACCUEIL",
                        "value": "Résu Accueil",
                        "date": None,
                    },
                ],
            },
        ],
        "context": "order",
        "sender_id": "middleware",
        "receiver_id": "SIL",
    }
    assert worklist == expected


def test_decode_order_comment(self):
    worklist = self.decode(
        """
    MSH|^~\\&|SIL||ONELINK||20210305095220||ORM^O33^ORM_O33|3653145742875247561|P|2.5.1|||AL|NE||UNICODE UTF-8|||
    PID|1|100202020150|||TESTGLIMS^Sebastien||20101220|M
    NTE|1|P|Anoréxie
    PV1|1||||||||||||||||||123456789
    SPM|1|121000007701|||||||||P||||||20210305092913|20210305080808
    ORC|NW|1210000077|121000007701
    NTE|1|P|Sous Immunosuppresseur
    OBR|1|1210000077|121000007701|CORPSCET||||||||||||0340^^^^^^^^^^^^^^^^^^^^^SERV&Nom service|0102030405||||||||||^^^20210305092900^^R
    OBR|2|1210000077|121000007701|NITRITES|||||||||||||||||||||||^^^20210305092900^^R
    SPM|1|121000007701|||||||||P||||||20210305092913|20210305080808
    ORC|CA|1210000077|121000007701
    OBR|3|1210000077|121000007701|UROBILIN|||||||||||||||||||||||^^^20210305092900^^R
    NTE|1|P|Comment OBR
    ORC|NW|1210000077|121000007701
    OBR|3|1210000077|121000007701|URONEW|||||||||||||||||||||||^^^20210305092900^^R
    """
    )
    expected = {
        "samples": [
            {
                "type": "patient",
                "sample_id": "121000007701",
                "request_id": "1210000077",
                "request_date": datetime.datetime(2021, 3, 5, 8, 8, 8),
                "patient_id": "100202020150",
                "last_name": "TESTGLIMS",
                "first_name": "Sebastien",
                "maiden_name": "",
                "birthdate": datetime.datetime(2010, 12, 20, 0, 0),
                "sex": "M",
                "location": None,
                "address": "",
                "postal_code": "",
                "city": "",
                "country": "",
                "phone": "",
                "room": "",
                "bed": "",
                "admission_date": None,
                "alternate_patient_id": "123456789",
                "collection_date": datetime.datetime(2021, 3, 5, 9, 29, 13),
                "comments": [
                    {"type": "sample", "text": "Sous Immunosuppresseur"},
                    {"type": "patient", "text": "Anoréxie"},
                    {"type": "sample", "text": "Comment OBR"},
                ],
                "orders": [
                    {"code": "CORPSCET", "priority": "R", "action": "add"},
                    {"code": "NITRITES", "priority": "R", "action": "add"},
                    {"code": "UROBILIN", "priority": "R", "action": "del"},
                    {"code": "URONEW", "priority": "R", "action": "add"},
                ],
                "physicians": [
                    {
                        "code": "0340",
                        "last_name": "",
                        "type": "prescriber",
                        "phone": "0102030405",
                    }
                ],
                "services": [
                    {"code": "SERV", "name": "Nom service", "phone": "0102030405"}
                ],
            }
        ],
        "context": "order",
        "sender_id": "SIL",
        "receiver_id": "ONELINK",
    }
    assert worklist == expected


def test_decode_result_other(self):
    worklist = self.decode(
        """
    MSH|^~\\&|SIL||cobasInfinity||20191112101611||OML^O33^OML_O33|SILORDER-65329-36970-000001|P|2.5|||AL|NE
    PID|1|0000077659|||TEST SIL^Connexion|SIL|20170207|M
    NTE|1|P||
    PV1|1||||||
    SPM|1|950000000003||ECBU
    ORC|NW|950000000003|950000000003
    OBR|1|9500000000|950000000003|ECBU^ECBU||||||||||||^RDF|||||||||||^^^20191112101607^^R
    NTE|1|P|Infos tube|"""
    )
    expected = {
        "context": "order",
        "sender_id": "SIL",
        "receiver_id": "cobasInfinity",
        "samples": [
            {
                "type": "patient",
                "sample_id": "950000000003",
                "request_id": "9500000000",
                "request_date": datetime.datetime(2019, 11, 12, 10, 16, 7),
                "patient_id": "0000077659",
                "last_name": "TEST SIL",
                "first_name": "Connexion",
                "maiden_name": "SIL",
                "birthdate": datetime.datetime(2017, 2, 7, 0, 0),
                "sex": "M",
                "location": None,
                "address": "",
                "postal_code": "",
                "city": "",
                "country": "",
                "phone": "",
                "room": None,
                "bed": None,
                "admission_date": None,
                "alternate_patient_id": None,
                "admission_date": None,
                "collection_date": datetime.datetime(2019, 11, 12, 10, 16, 7),
                "comments": [{"type": "sample", "text": "Infos tube"}],
                "orders": [{"code": "ECBU", "priority": "R", "action": "add"}],
            }
        ],
    }
    assert worklist == expected


def test_decode_without_demande_id(self):
    try:
        worklist = self.decode(
            """
        MSH|^~\\&|SIL||cobasInfinity||20191112101611||OML^O33^OML_O33|SILORDER-65329-36970-000001|P|2.5|||AL|NE
        PID|1|0000077659|||TEST SIL^Connexion|SIL|20170207|M
        NTE|1|P||
        PV1|1||||||
        SPM|1|950000000003||ECBU
        ORC|NW|950000000003|950000000003
        OBR|1||950000000003|ECBU^ECBU||||||||||||^RDF|||||||||||^^^20191112101607^^R
        NTE|1|P|Infos tube|"""
        )
    except RuntimeError:
        worklist = None
    assert worklist is None


def test_encode_result(self):
    result_worklist = copy.deepcopy(self.result_worklist)
    result_worklist.samples[0].collection_date = datetime.datetime(
        2013, 5, 13, 10, 40, 3
    )
    result_worklist.samples[0].results[0].comments = ["Comment 1", "Comment 2"]
    result_worklist.samples[0].results.add(
        {
            "type": "GC",
            "code": "ELECGR",
            "value": [
                {"value": 3, "label": None, "codification": None},
                {"value": 2, "label": None, "codification": None},
                {"value": 1, "label": None, "codification": None},
            ],
            "date": datetime.datetime(2021, 2, 25, 15, 40),
        }
    )
    result_worklist.samples[0].results.add(
        {
            "status": "F",
            "code": "T1",
            "lots": [],
            "value": [{"codification": "L", "value": "TCODE", "label": "Texte Code"}],
            "instrument": "Instrument",
            "validator": "Nom Prenom",
            "validator_code": "login",
            "operator": "OPE",
            "comments": [],
            "type": "CE",
            "unit": None,
            "date": datetime.datetime(2021, 2, 25, 15, 40),
        }
    )

    message = self.result(
        worklist=result_worklist, context={"control_id": "1521651964637"}
    )
    expected = """
    MSH|^~\\&|MCA||||20130715141912||OUL^R22^OUL_R22|1521651964637||2.5.1|||AL|NE||UNICODE UTF-8
    PID|1|036666666|036666666||NYME^ANO|NJF|20110513|F
    PV1|1||LOC||||ATTENDING^Joseph-Louis|PRESCRIBER|||||||||||ADM_CLI|||||||||||||||||||||||||20130511103030|20130514103001
    SPM|1|123456|||||||||P||||||20130513104003|20130603110342
    OBR|1|78910|123456|A|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|NM|A||42|mV|||||P|||20110506121400||VAL||INST
    NTE|1|L|Comment 1
    NTE|2|L|Comment 2
    OBR|2|78910|123456|B|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|CE|B||NORESULT|mV||HH~>|||P|||20110506121400||VAL||INST
    OBR|3|78910|123456|C|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|NM|C||42|mV|||||P|||20110506121400||VAL||INST
    NTE|1|L|Abnormal
    NTE|2|L|EE
    OBR|4|78910|123456|ELECGR|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|GC|ELECGR||3~2~1||||||P|||20210225154000
    OBR|5|78910|123456|T1|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003|||||||OPE
    OBX|1|CE|T1||TCODE^Texte Code^L||||||P|||20210225154000||Nom Prenom||Instrument
    """
    assert message == expected
    self.configure(option_alarme_analytique=True)
    message = self.result(
        worklist=result_worklist, context={"control_id": "1521651964637"}
    )
    expected = """
    MSH|^~\\&|MCA||||20130715141912||OUL^R22^OUL_R22|1521651964637||2.5.1|||AL|NE||UNICODE UTF-8
    PID|1|036666666|036666666||NYME^ANO|NJF|20110513|F
    PV1|1||LOC||||ATTENDING^Joseph-Louis|PRESCRIBER|||||||||||ADM_CLI|||||||||||||||||||||||||20130511103030|20130514103001
    SPM|1|123456|||||||||P||||||20130513104003|20130603110342
    OBR|1|78910|123456|A|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|NM|A||42|mV|||||P|||20110506121400||VAL||INST
    NTE|1|L|Comment 1
    NTE|2|L|Comment 2
    OBR|2|78910|123456|B|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|CE|B||NORESULT|mV||LL~<|||P|||20110506121400||VAL||INST
    OBR|3|78910|123456|C|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|NM|C||42|mV|||||P|||20110506121400||VAL||INST
    NTE|1|L|Abnormal
    NTE|2|L|EE
    OBR|4|78910|123456|ELECGR|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003
    OBX|1|GC|ELECGR||3~2~1||||||P|||20210225154000
    OBR|5|78910|123456|T1|||||||||Information clinique|||PRESCRIBER^Prescriber|||||||||||^^^20130513104003|||||||OPE
    OBX|1|CE|T1||TCODE^Texte Code^L||||||P|||20210225154000||Nom Prenom||Instrument
    """


def test_encode_control(self):
    result_control_worklist = copy.deepcopy(self.result_control_worklist)
    result_control_worklist.samples[0].specimen = "U"
    message = self.result_control(
        worklist=result_control_worklist, context={"control_id": "1521651964637"}
    )
    expected = """
    MSH|^~\\&|MCA||||20130715141912||OUL^R22^OUL_R22|1521651964637||2.5.1|||AL|NE||UNICODE UTF-8
    SPM|1|CQ123456|||||||||QC
    SAC|1||CQ123456
    OBR|1||CQ123456|A
    OBX|1|NM|A||42|mV||HH~>|||P|||20110506121400||||INST
    OBR|2||CQ123456|B
    OBX|1|NM|B||4.40||||||P|||20110506121300||||INST
    INV|Reagent Name||reagent|REAGENT||||||||20120606120000
    INV|2||control|CONTROL
    INV|3||calibration|CALIBRATION
    SPM|2|CQ123488|||||||||QC
    SAC|1||CQ123488
    OBR|1||CQ123488|A
    OBX|1|NM|A||4|mV|||||P|||20110506121400||||INST
    """
    assert message == expected
