# -*- coding: utf8 -*-

from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "immulite"


def test_query(self):
    result = self.decode(
        """
    H|\\^&||||111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|||P|1|20140602113748
    Q|1|^10602516215||ALL||||||||O
    L|1"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "10602516215", "type": "patient"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^&||||111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|||P|1|20140602113748
    Q|1|^10602516215||ALL||||||||O
    L|1"""
            )
        },
    )
    expected = """
    H|\\^&|||WST|||||WST||P|1|20130715141912
    L|1|F"""
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||WST|||||WST||P|1|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F
        O|1|123456||^^^A^1:10\\^^^B|A||20130513114242
        L|1|F"""
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&||||111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|||P|1|20130402130105
    P|1|011092446738|||ANO^NYME||19500727|F|||||
    O|1|10330004615||^^^CAL|R||||||||Normal|||||||||||G1902
    R|1|^^^CAL|<2.00|pg/mL|2.00\\2.00^18.2\\2000|<|N|F||OPE|20130402115103|20130402130105|G1902
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "10330004615",
                "type": "patient",
                "results": [
                    {
                        "flags": ["<"],
                        "date": datetime.datetime(2013, 4, 2, 13, 1, 5),
                        "value": "<2.00",
                        "code": "CAL",
                        "operator": "OPE",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc_result(self):
    result = self.decode(
        """
    H|\\^&||||111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|||P|1|20130403112604
    P|1|030|||CMMNEG||20150131||||||1
    O|1|~CCMMNEG0302015011||^^^CMM|R|||||||||||1||308^309^310||||||G1902
    R|1|^^^CMM|0.057||0.0\\0.0^0.60\\50.0|N|N|F|||20130403094230|20130403112604|G1902
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "CMMNEG0302015011",
                "control_lot": "CMMNEG0302015011",
                "type": "control",
                "results": [
                    {
                        "date": datetime.datetime(2013, 4, 3, 11, 26, 4),
                        "value": "0.057",
                        "code": "CMM",
                        "flags": [],
                        "operator": "",
                        "lots": [{"lot": "308", "type": "reagent"}],
                    }
                ],
            }
        ],
    }
    assert result == expected
