# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime


parser_class = "pentra_c200"


def test_order(self):
    worklist = self.encode(
        self.order_worklist,
        context={
            "query": self.parse(
                """
        H|\\^&|||Analyzer|||||||||20010111055300
        Q|1|910000000001||||||||||N
        L|1
        """
            )
        },
    )
    expected = """
        H|\\^&|||MCA|||||||||20130715141912
        P|1|036666666|||NYME^ANO||20110513|F|||||PRESCRIBER
        O|1|910000000001||^^^A\\^^^B\\^^^C|||||||||||Serum
        L|1
    """
    assert worklist == expected


def test_decode_query(self):
    result = self.decode(
        """
        H|\\^&|||Analyzer|||||||||20010111055300
    Q|1|910000000001||||||||||N
    L|1
    """
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "910000000001", "type": "patient"}],
    }
    assert result == expected


def test_decode_results(self):
    worklist = self.decode(
        """
        H|\\^&|||Analyzer|||||||||20010111055300
    P|1|PID2734|||Last^Middle^First||19630501|M|Race1||||AttenPhID|1234567890123
    O|1|910000000001||^^^1
    C|1|I|Order Comment|G
    R|1|^^^1|15.265|mg/ml||H||||||20010110121530
    C|1|I|Result Comment|G
    L|1
    """
    )
    date = datetime.datetime(2001, 1, 10, 12, 15, 30)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "1",
                        "comments": ["Order Comment", "Result Comment"],
                        "date": date,
                        "flags": ["H"],
                        "unit": "mg/ml",
                        "value": "15.265",
                    }
                ],
                "sample_id": "910000000001",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(pentra_c200_qc_prefix="QC")
    worklist = self.decode(
        """
        H|\\^&|||Analyzer|||||||||20010111055300
    P|1|PID2734|||Last^Middle^First||19630501|M|Race1||||AttenPhID|1234567890123
    O|1|QC123456||^^^1
    R|1|^^^1|15.265|mg/ml||1||||||20010110121530
    C|1|I|CQ Comment|G
    L|1
    """
    )
    date = datetime.datetime(2001, 1, 10, 12, 15, 30)
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "QC123456",
                "results": [
                    {
                        "code": "1",
                        "comments": ["CQ Comment"],
                        "date": date,
                        "flags": ["1"],
                        "unit": "mg/ml",
                        "value": "15.265",
                    }
                ],
                "sample_id": "QC123456",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
