# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.tests import BaseTest
from driverng import utils


class PreviisolaxmlTest(BaseTest):
    MAPPING_CLASS = "previisolaxml"

    @staticmethod
    def unindent(data):
        data = "".join(
            [li.lstrip().replace("\t", "") for li in data.splitlines() if li.strip()]
        )
        return utils.prettify_xml(data.encode("utf8"))

    def test_order(self):
        self.assertOrderMessageEqual(
            self.WORKLIST_NEW_ADD_DEL,
            """
            <?xml version="1.0" encoding="UTF-8"?>
            <lisMessage xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="ASTM.xsd">
            <header>
                <version>1394-97</version>
                <dateTime>20130715141912</dateTime>
            </header>
            <request>
            <patientInformation>
                <patientIdentifier>036666666</patientIdentifier>
                <lastName>NYME</lastName>
                <firstName>ANO</firstName>
                <birthDate>20110513</birthDate>
                <sex>F</sex>
                <location>SER</location>
            </patientInformation>
            <testOrder>
                <specimen>
                    <specimenIdentifier>123456</specimenIdentifier>
                    <collectedDateTime>20130513114242</collectedDateTime>
                    <specimenType>A</specimenType>
                </specimen>
                <test>
                    <universalIdentifier>
                        <testIdentifier>CULTURE</testIdentifier>
                    </universalIdentifier>
                </test>
            </testOrder>
            <testOrder>
                <specimen>
                    <specimenIdentifier>123456</specimenIdentifier>
                    <collectedDateTime>20130513114242</collectedDateTime>
                    <specimenType>B</specimenType>
                </specimen>
                <test>
                    <universalIdentifier>
                        <testIdentifier>CULTURE</testIdentifier>
                    </universalIdentifier>
                </test>
            </testOrder>
            </request>
            </lisMessage>
            """,
        )

    def test_resultat(self):
        date = datetime.datetime(2010, 5, 24, 11, 51, 1)
        self.assertResultEqual(
            """
        <aiMessage xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="ASTM.xsd">
        <header>
            <version>1394-97</version>
            <dateTime>20100524115101</dateTime>
            <instrumentName>Isola</instrumentName>
        </header>
        <requestResult>
            <patientInformation>
                <patientIdentifier>P111444</patientIdentifier>
                <lastName>Moor</lastName>
                <firstName>Kevin</firstName>
                <birthDate>20050925</birthDate>
                <sex>M</sex>
                <location/>
            </patientInformation>
            <testOrder>
                <specimen>
                    <specimenIdentifier>1012999</specimenIdentifier>
                    <collectedDateTime>20100524095000</collectedDateTime>
                    <specimenType>ECBU</specimenType>
                    <specimenSource/>
                    <relevantClinicalInformation/>
                </specimen>
                <test>
                    <universalIdentifier>
                        <testIdentifier>CULTURE</testIdentifier>
                        <testName>URINES</testName>
                    </universalIdentifier>
                    <result>
                        <value/>
                        <resultStatus>F</resultStatus>
                        <operatorName/>
                        <resultDateTime>20100524115101</resultDateTime>
                        <bmxResultData>
                            <disposable>
                                <disposableType>PPM</disposableType>
                                <reference>CPS3</reference>
                                <disposableIdentifier/>
                                <disposableRelevantInformation>AE37</disposableRelevantInformation>
                                <disposableNumber>1/1</disposableNumber>
                                <lotNumber>123456789</lotNumber>
                                <expirationDate>20101223</expirationDate>
                            </disposable>
                            <disposable>
                                <disposableType>TIP</disposableType>
                                <reference/>
                                <disposableIdentifier/>
                                <disposableRelevantInformation/>
                                <disposableNumber/>
                                <lotNumber>789456123</lotNumber>
                                <expirationDate>20110124</expirationDate>
                            </disposable>
                            <disposable>
                                <disposableType>APPLICATOR</disposableType>
                                <reference/>
                                <disposableIdentifier/>
                                <disposableRelevantInformation/>
                                <disposableNumber/>
                                <lotNumber>456789123</lotNumber>
                                <expirationDate>20110224</expirationDate>
                            </disposable>
                        </bmxResultData>
                    </result>
                </test>
            </testOrder>
        </requestResult>
    </aiMessage>""",
            {
                "1012999": {
                    "resultats": [
                        {
                            "date": date,
                            "resu": "CPS3",
                            "test": "BOITE_ECBU",
                            "commentaires": [
                                "CPS3 (1): lot 123456789 expiration 20101223. AE37 PPM"
                            ],
                        },
                        {
                            "date": date,
                            "resu": "APPLICATOR;TIP",
                            "test": "INFO_ECBU",
                            "commentaires": [
                                "TIP: lot 789456123 expiration 20110124",
                                "APPLICATOR: lot 456789123 expiration 20110224",
                            ],
                        },
                        {
                            "date": date,
                            "resu": "APPLICATOR;CPS3;TIP",
                            "test": "CULTURE_ECBU",
                            "commentaires": [
                                "CPS3 (1): lot 123456789 expiration 20101223. AE37 PPM",
                                "TIP: lot 789456123 expiration 20110124",
                                "APPLICATOR: lot 456789123 expiration 20110224",
                            ],
                        },
                        {
                            "resu": "20100524115101",
                            "test": "CULTURE_ECBU_RESULTDATETIME",
                        },
                    ]
                }
            },
        )

    def test_plating_date(self):
        date = datetime.datetime(2021, 9, 16, 11, 40, 29)
        self.assertResultEqual(
            """
        <?xml version="1.0" encoding="UTF-8"?>
        <aiMessage xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:noNamespaceSchemaLocation="ASTM.xsd">
        <header>
            <version>1394-97</version>
            <dateTime>20210916114029</dateTime>
            <instrumentName>Isola</instrumentName>
        </header>
        <requestResult>
            <patientInformation>
                <patientIdentifier>C1091402644</patientIdentifier>
                <lastName>ESSAI MCA PREVI ISOLA</lastName>
                <firstName></firstName>
                <birthDate>20010101</birthDate>
                <sex>F</sex>
                <location></location>
            </patientInformation>
            <testOrder>
                <specimen>
                    <specimenIdentifier>330925917012</specimenIdentifier>
                    <collectedDateTime>20210916101400</collectedDateTime>
                    <specimenType>ECBU</specimenType>
                    <specimenSource></specimenSource>
                    <relevantClinicalInformation></relevantClinicalInformation>
                </specimen>
                <test>
                    <universalIdentifier>
                        <testIdentifier>CULTURE</testIdentifier>
                        <testName></testName>
                        </universalIdentifier>
                    <result>
                        <value/>
                        <resultStatus>F</resultStatus>
                        <operatorName></operatorName>
                        <resultDateTime>20210916114029</resultDateTime>
                        <bmxResultData></bmxResultData>
                    </result>
                </test>
            </testOrder>
        </requestResult>
        </aiMessage>
        """,
            {
                "330925917012": {
                    "resultats": [
                        {
                            "resu": "",
                            "test": "BOITE_ECBU",
                            "date": date,
                            "commentaires": [""],
                        },
                        {
                            "resu": "",
                            "test": "CULTURE_ECBU",
                            "date": date,
                            "commentaires": [""],
                        },
                        {
                            "resu": "20210916114029",
                            "test": "CULTURE_ECBU_RESULTDATETIME",
                        },
                    ]
                }
            },
        )
