from __future__ import unicode_literals

import datetime

parser_class = "uwam"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||||||||||LIS2-A2
    P|1|036666666|L036666666||NYME^ANO||20110513|F
    O|1|123456||^^^A|A||20130513114242||||N||||Serum|PRESCRIBER^Prescriber|||||||||O
    O|2|123456||^^^B|A||20130513114242||||A||||Serum|PRESCRIBER^Prescriber|||||||||O
    O|3|123456||^^^C|A||20130513114242||||C||||Serum|PRESCRIBER^Prescriber|||||||||O
    L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||U-WAM^00-08_Build008^11001^^^^AU501736||||||||LIS2-A2|20170307144247
    Q|1|20150730001||||20170307144247||||||C
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "20150730001"}],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2018, 7, 23, 11, 41, 42)
    worklist = self.decode(
        """
    H|\\^&|||U-WAM^00-08_Build008^11001^^^^AU501736||||||||LIS2-A2|20170307144247
    P|1||P00001||Sample^Ichiro^^Sample^Ichiro^^^||19701130|M||||||ANEG||||||||||IP
    O|1|12345678901234^AA^01||^^^C-PH^^^C-LEU^^^C-NIT^^^C-PRO^^^C-GLU^^^C-KET^^^C-URO^^^C-BIL^^^C-COLOR^^^C-ColorRANK^^^C-CLOUD^^^C-CRE^^^C-ALB^^^C-P/C^^^C-A/C^^^C-S.G.(Ref)^^^C-BLD|R||20151211180241||||N|||20151211180247|Urine||||||||||
    R|1|^^^C-PH^A^1^S^0002^01|7.5^RAW|||N||||^^admin^administrator||20180723114142|UC_3500
    R|2|^^^C-PH^A^1^S^0002^01|7.5^MAINFORMAT|||N||||^^admin^administrator||20180723114142|UC_3500
    R|3|^^^C-LEU^A^1^S^0002^01|500^RAW|/\xcel||N||||^^admin^administrator||20180723114142|UC_3500
    R|4|^^^C-LEU^A^1^S^0002^01|3+^MAINFORMAT|||N||||^^admin^administrator||20180723114142|UC_3500
    R|5|^^^C-URO^A^1^S^  0022^08|^RAW|mg/dL||N||||^^Iryna^Iryna||20180723114142|UC-3500
    R|6|^^^C-URO^A^1^S^  0022^08|^MAINFORMAT|mg/dL||N||||^^Iryna^Iryna||20180723114142|UC_3500
    R|14|^^^CW_SSH_AxCW_FSC_W-BF^A^1^IF|20150827&R&PNG&R&[UF-5000&S&11001]&[20150827_121025]&R&[2015082700000001]_[CW_SSH_AxCW_FSC_W].png^RAW|||||||^^device||20180723114142|UC_3500
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "12345678901234",
                "specimen": "Urine",
                "rack": "AA-01",
                "results": [
                    {
                        "code": "C-PHRAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "UC_3500",
                        "operator": "admin",
                        "value": "7.5",
                        "date": date,
                    },
                    {
                        "code": "C-PHMAINFORMAT",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "UC_3500",
                        "operator": "admin",
                        "value": "7.5",
                        "date": date,
                    },
                    {
                        "code": "C-LEURAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "UC_3500",
                        "operator": "admin",
                        "value": "500",
                        "date": date,
                    },
                    {
                        "code": "C-LEUMAINFORMAT",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "UC_3500",
                        "operator": "admin",
                        "value": "3+",
                        "date": date,
                    },
                    {
                        "code": "CW_SSH_AxCW_FSC_W-BF",
                        "comments": [],
                        "date": date,
                        "dilution": "1",
                        "flags": [""],
                        "instrument": "UC_3500",
                        "operator": "device",
                        "value": "[2015082700000001]_[CW_SSH_AxCW_FSC_W].png",
                    },
                    {"code": "CRISTAUX", "value": "ABS", "instrument": "UC_3500"},
                    {"code": "CYLINDRES", "value": "ABS", "instrument": "UC_3500"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_with_cristaux(self):
    self.configure(
        uwam_cristaux="UAMOR;PAMOR;OXCAL;ACUR;PHOSCA;PAM;URAMO;CARCA;BILI;CYST;CHOL;TYRO;LEUC",
        uwam_cylindres="CYLEP;CYLGRA;CYLHYA;CYLRBC;CYLWBC",
    )
    date = datetime.datetime(2018, 8, 21, 15, 0, 48)
    worklist = self.decode(
        """
    H|\\^&|||U-WAM^00-13_Build003^A1710^^^^AU501736||||||||LIS2-A2|20180821150048
    P|1||5468||Z_LABO^TEST||19880508|F
    O|1|17||^^^RBC\\^^^WBC\\^^^EC\\^^^BACT\\^^^YLC\\^^^UAMOR\\^^^PAMOR\\^^^OXCAL\\^^^ACUR\\^^^PAM\\^^^SPERM\\^^^CAST\\^^^XTAL\\^^CHOL\\|R||20180821143628||||N|||20180821143707|*||||||||||F
    R|1|^^^RBC^M^1^S|<1^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|2|^^^WBC^M^1^S|128.5^MAINFORMAT|ul||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|3|^^^UAMOR^M^1^S|QQS^RAW|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|4|^^^UAMOR^M^1^S|QQS^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|5|^^^PAMOR^M^1^S|NB^RAW|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|6|^^^PAMOR^M^1^S|NB^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|7|^^^OXCAL^M^1^S|RR^RAW|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|8|^^^OXCAL^M^1^S|RR^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|9|^^^ACUR^M^1^S|NB^RAW|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|10|^^^ACUR^M^1^S|NB^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|11|^^^PAM^M^1^S|ANB^RAW|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|12|^^^PAM^M^1^S|ANB^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|13|^^^SPERM^M^1^S|PRES^RAW|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|14|^^^SPERM^M^1^S|PRES^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|15|^^^CAST^M^1^S|0.00^RAW|/ul||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|16|^^^CAST^M^1^S|ABS^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|17|^^^XTAL^M^1^S|651.9^RAW|/ul||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|18|^^^XTAL^M^1^S|NB^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|19|^^^CHOL^M^1^S|QQS^RAW|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    R|20|^^^CHOL^M^1^S|QQS^MAINFORMAT|||N||||admin^administrator^admin^administrator||20180821150048|ENTERED
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "17",
                "specimen": "*",
                "rack": "",
                "results": [
                    {
                        "code": "RBCMAINFORMAT",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "<1",
                        "date": date,
                    },
                    {
                        "code": "WBCMAINFORMAT",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "128.5",
                        "date": date,
                    },
                    {
                        "code": "SPERMRAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "PRES",
                        "date": date,
                    },
                    {
                        "code": "SPERMMAINFORMAT",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "PRES",
                        "date": date,
                    },
                    {
                        "code": "CASTRAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "0.00",
                        "date": date,
                    },
                    {
                        "code": "CASTMAINFORMAT",
                        "comments": [],
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "ABS",
                        "date": date,
                    },
                    {
                        "code": "XTALRAW",
                        "comments": [],
                        "date": datetime.datetime(2018, 8, 21, 15, 0, 48),
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "651.9",
                    },
                    {
                        "code": "XTALMAINFORMAT",
                        "comments": [],
                        "date": datetime.datetime(2018, 8, 21, 15, 0, 48),
                        "dilution": "1",
                        "flags": ["N"],
                        "instrument": "ENTERED",
                        "operator": "admin",
                        "value": "NB",
                    },
                    {
                        "code": "CRISTAUX",
                        "value": "UAMORQQS;PAMORNB;OXCALRR;ACURNB;PAMANB;CHOLQQS",
                        "instrument": "ENTERED",
                    },
                    {"code": "CYLINDRES", "value": "ABS", "instrument": "ENTERED"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_scientific_segment(self):
    worklist = self.decode(
        """
    H|\\^&|||U-WAM^00-09_Build006^A1219^^^^AU501736||||||||LIS2-A2|20180801094032
    S|1|||||[UD-10]_[QCTargetLimit]_[20180801_094032]_[QC27]_[UF control -H]_[UK0043].csv
    L|1|N
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2015, 8, 4, 10, 37, 35)
    worklist = self.decode(
        """
    H|\\^&|||U-WAM^00-08_Build008^11001^^^^AU501736||||||||LIS2-A2|20170307144247
    P|1
    O|1|QC-S1504A||^^^RBC^^^WBC^^^EC^^^CAST^^^BACT^^^Cond.|||||||Q||||||||||||||F
    R|1|^^^RBC^^1^S|41.8^RAW|\xcel||||||||20150804103735|11001
    R|2|^^^WBC^^1^S|40.9^RAW|\xcel||||||||20150804103735|11001
    R|3|^^^EC^^1^S|6.7^RAW|\xcel||||||||20150804103735|11001
    R|4|^^^CAST^^1^S|1.96^RAW|\xcel||||||||20150804103735|11001
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "QC-S1504A",
                "control_lot": "QC-S1504A",
                "results": [
                    {
                        "code": "RBCRAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": [""],
                        "instrument": "11001",
                        "operator": "",
                        "value": "41.8",
                        "date": date,
                    },
                    {
                        "code": "WBCRAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": [""],
                        "instrument": "11001",
                        "operator": "",
                        "value": "40.9",
                        "date": date,
                    },
                    {
                        "code": "ECRAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": [""],
                        "instrument": "11001",
                        "operator": "",
                        "value": "6.7",
                        "date": date,
                    },
                    {
                        "code": "CASTRAW",
                        "comments": [],
                        "dilution": "1",
                        "flags": [""],
                        "instrument": "11001",
                        "operator": "",
                        "value": "1.96",
                        "date": date,
                    },
                    {"code": "CRISTAUX", "value": "ABS", "instrument": "11001"},
                    {"code": "CYLINDRES", "value": "ABS", "instrument": "11001"},
                ],
            }
        ],
    }
    assert worklist == expected
