# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

from driverng import Worklist

parser_class = "versacell"


def test_order(self):
    result = self.order()
    expected = """
        H|\\^&|||MCA||||N81|SIEMENS||P|1|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F
        O|1|123456||^^^A\\^^^B|A|20130603110342|20130513114242
        L|1|N
        """
    assert result == expected


def test_query(self):
    result = self.decode(
        """
        H|\\^&|||SIEMENS|111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|MCA||P|1|20131218153346
        Q|1|^652112187911||ALL||||||||O
        L|1"""
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "652112187911", "type": "patient"}],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^&|||SIEMENS|111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|MCA||P|1|20131218153346
    Q|1|^652112187911||ALL||||||||O
    L|1"""
            )
        },
    )
    expected = """
    H|\\^&|||MCA||||N81|SIEMENS||P|1|20130715141912
    Q|1|^652112187911||ALL||||||||X
    L|1|I"""
    assert result == expected


def test_result(self):
    date = datetime.datetime(2013, 12, 18, 15, 17, 26)
    result = self.decode(
        """
    H|\\^&|||SIEMENS|111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|MCA||P|1|20131218153222
    P|1|0113011767799|||ANO^NYME||19341220|M|||||
    O|1|650212187918||^^^LIPE^^NEAT|||20131218103000||||0||||1|||||||||ADVIA 1800|
    R|1|^^^LIPE^^NEAT|0||||N|F||||20131218151726|ADVIA 1800
    P|2|0113011767799|||ANO^NYME||19341220|M|||||
    O|1|650212187918||^^^HEMO^^NEAT|||20131218103000||||0||||1|||||||||ADVIA 1800|
    R|1|^^^HEMO^^NEAT|0||||N|F||||20131218151726|ADVIA 1800
    P|3|0113011767799|||ANO^NYME||19341220|M|||||
    O|1|650212187918||^^^ICT^^NEAT|||20131218103000||||0||||1|||||||||ADVIA 1800|
    R|1|^^^ICT^^NEAT|0||||N|F||||20131218151726|ADVIA 1800
    P|4|0113011767799|||ANO^NYME||19341220|M|||||
    O|1|650212187918||^^^NA^^NEAT|||20131218103000||||0||||1|||||||||ADVIA 1800|
    R|1|^^^NA^^NEAT|135.3||||N|F||||20131218151726|ADVIA 1800
    P|5|0113011767799|||ANO^NYME||19341220|M|||||
    O|1|650212187918||^^^K^^NEAT|||20131218103000||||0||||1|||||||||ADVIA 1800|
    R|1|^^^K^^NEAT|5.38||||N|F||||20131218151726|ADVIA 1800
    P|6|0113011767799|||ANO^NYME||19341220|M|||||
    O|1|650212187918||^^^CL^^NEAT|||20131218103000||||0||||1|||||||||ADVIA 1800|
    R|1|^^^CL^^NEAT|97.0||||N|F||||20131218151726|ADVIA 1800
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "650212187918",
                "type": "patient",
                "results": [
                    {
                        "instrument": "ADVIA 1800",
                        "date": date,
                        "value": "0",
                        "code": "LIPE",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "instrument": "ADVIA 1800",
                        "date": date,
                        "value": "0",
                        "code": "HEMO",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "instrument": "ADVIA 1800",
                        "date": date,
                        "value": "0",
                        "code": "ICT",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "instrument": "ADVIA 1800",
                        "date": date,
                        "value": "135.3",
                        "code": "NA",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "instrument": "ADVIA 1800",
                        "date": date,
                        "value": "5.38",
                        "code": "K",
                        "comments": [],
                        "flags": [],
                    },
                    {
                        "instrument": "ADVIA 1800",
                        "date": date,
                        "value": "97.0",
                        "code": "CL",
                        "comments": [],
                        "flags": [],
                    },
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^&|||SIEMENS|111 Canfield Ave^Randolph^NJ^07869||(201)927-2828|N81|MCA||P|1|20131218141711
    P|1|Multiqual 2|||45652     /1404||||||||
    O|1|PB006||^^^LIPA^^NEAT|||20131218||||Q||||1|||||||||ADVIA 1800|
    R|1|^^^LIPA^^NEAT|55||||N|F||||20131218135500|ADVIA 1800
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "45652",
                "type": "control",
                "control_lot": "45652",
                "specimen": "1",
                "results": [
                    {
                        "instrument": "ADVIA 1800",
                        "date": datetime.datetime(2013, 12, 18, 13, 55),
                        "value": "55",
                        "code": "LIPA",
                        "comments": [],
                        "flags": [],
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_mode_ascii(self):
    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0]["last_name"] = "éçà"
    worklist.samples[0]["first_name"] = "îïÖ"

    result = self.order(worklist=worklist)
    expected = """
    H|\\^&|||MCA||||N81|SIEMENS||P|1|20130715141912
    P|1|036666666|||éçà^îïÖ||20110513|F
    O|1|123456||^^^A\\^^^B|A|20130603110342|20130513114242
    L|1|N"""
    assert result == expected
    self.configure(mode_ascii=True)
    result = self.order(worklist=worklist)
    expected = """
    H|\\^&|||MCA||||N81|SIEMENS||P|1|20130715141912
    P|1|036666666|||eca^iiO||20110513|F
    O|1|123456||^^^A\\^^^B|A|20130603110342|20130513114242
    L|1|N"""
    assert result == expected
